#include "Thermometer.h"
#ifdef ENABLE_THERMOMETER

#include <Source/Core/Settings.h>
#include <Source/Core/Utils.h>


void CWndThermometer::Create(CWnd *pParent, ui16 dwFlags)
{
	CWnd::Create("CWndCharMap", dwFlags, CRect(0, 16, 400, 240), pParent);
}

void DrawTemp(int nTemp)
{
	int nY = 160-(154-34)*nTemp/50;
	CRect rcBar( 50-11-2, 34, 50+13+1, 190-10 );
	for ( int y = rcBar.top; y < rcBar.bottom; y++ )
		for ( int x = rcBar.left; x < rcBar.right; x++ )
		{
			ui16 pix = BIOS::LCD::GetPixel(x, y);
			int nR = Get565R(pix);
			int nG = Get565G(pix);
			int nBlk = max( nR, nG );

			nBlk |= 7;
			if ( y < nY ) 
				pix = CUtils::InterpolateColor( RGB565(000000), RGB565(ffffff), nBlk );
			else
				pix = CUtils::InterpolateColor( RGB565(000000), RGB565(ff0000), nBlk );
			BIOS::LCD::PutPixel(x, y, pix);
		}
}

void CWndThermometer::OnTimer()
{
	static int nTemp = 0;
	nTemp += 1;
	if (nTemp > 50)
		nTemp = 0;
	DrawTemp( nTemp );
//	Invalidate();
}

void CWndThermometer::OnPaint()
{
	BIOS::LCD::Bar( m_rcClient, RGB565(ffffff) );
	CBitmap bmp;
	bmp.Load( bitmapThermometer );
	for ( int i=0; i<16; i++ )
	{
		ui16& clr = bmp.m_arrPalette[i];
		if ( clr == RGB565(ffffff) )
		{
			clr = RGB565(ff00ff);
			continue;
		}
		int r = Get565R( clr );
		//int g = Get565G( clr );
		//int b = Get565B( clr );
		clr = RGB565RGB( r, r, r );
	}

	bmp.Blit ( 20, 21 );

	for ( int x = -24; x <= 24; x++ )
		for ( int y = -24; y <= 24; y++ )
		{
			if ( x*x + y*y > 24*24 )
				continue;
			if ( x*x + y*y < 22*22 )
			{
				BIOS::LCD::PutPixel( 50+x, 206+y, RGB565(ff0000) );	
				continue;
			}
			int l = CUtils::Sqrt( ((x*x + y*y) - 22*22)*25*25 );

			UTILS.Clamp(l, 0, 255);
			ui16 pix = RGB565(ffffff);
			pix = CUtils::InterpolateColor( RGB565(ff0000), pix, l );
			BIOS::LCD::PutPixel( 50+x, 206+y, pix );
		}


	CRect rcBar( 37, 180, 65, 190 );
	for ( int y = rcBar.top; y < rcBar.bottom; y++ )
		for ( int x = rcBar.left; x < rcBar.right; x++ )
		{
			ui16 pix = BIOS::LCD::GetPixel(x, y);
			int nBlk = Get565R(pix);
			nBlk |= 7;
			pix = CUtils::InterpolateColor( RGB565(000000), RGB565(ff0000), nBlk );
			BIOS::LCD::PutPixel(x, y, pix);
		}

	for ( int i=0; i<=50; i+=10 )
		BIOS::LCD::Printf( 70,  154-(154-34)*i/50, RGB565(b0b0b0), RGB565(ffffff), "%2d\xf8" "C", i );
	
	DrawTemp(0);
}

void CWndThermometer::OnKey(ui16 nKey)
{
}

void CWndThermometer::OnMessage(CWnd* pSender, ui16 code, ui32 data)
{
	// LAYOUT ENABLE/DISABLE FROM TOP MENU BAR
	if (code == ToWord('L', 'D') )
	{
		KillTimer();
		return;
	}

	if (code == ToWord('L', 'E') )
	{
		SetTimer(500);
		return;
	}
}


LINKERSECTION(".extra")
/*static*/ const unsigned char CWndThermometer::bitmapThermometer[] = {
	0x47, 0x42, 0x3a, 0xd7, 0x01, 0x00, 0x1f, 0x00, 0x1e, 0x00, 0x18, 0x00, 0x3e, 0x10, 0x9e, 0x10, 0x97, 0x08, 0x70, 0x08, 0x49, 0x08, 0x41, 0xff, 0xff, 0xf7, 0x9e, 0xde, 0xfb, 0xb5, 0x96, 0x94, 
	0x92, 0x63, 0x0c, 0x39, 0xc7, 0x83, 0x93, 0xac, 0xef, 0x38, 0x3f, 0xec, 0xaf, 0x49, 0x1a, 0xd1, 0x81, 0x77, 0x36, 0x07, 0x18, 0x2f, 0xda, 0xb5, 0x95, 0xac, 0xf8, 0x75, 0x74, 0x45, 0x78, 0xfc, 
	0xa4, 0x93, 0xd8, 0x75, 0xb1, 0x40, 0x61, 0x81, 0xda, 0xe4, 0x91, 0xbd, 0x18, 0x05, 0xe1, 0x42, 0x78, 0xde, 0x39, 0x2c, 0x87, 0x82, 0x40, 0x51, 0x81, 0xdb, 0xa4, 0x94, 0xac, 0xf8, 0x5a, 0x24, 
	0x27, 0x8b, 0xb3, 0x92, 0xe8, 0x5b, 0x24, 0x46, 0x8e, 0xba, 0xf3, 0x91, 0xbd, 0x18, 0x04, 0xc2, 0x42, 0x68, 0xb9, 0x39, 0x1e, 0x8e, 0x24, 0x46, 0x8e, 0xca, 0xd3, 0x94, 0xbd, 0xf8, 0x5e, 0x24, 
	0x27, 0xfa, 0xf2, 0x92, 0xc8, 0x5f, 0x24, 0x37, 0x8d, 0xbc, 0x39, 0x4a, 0xce, 0x86, 0xf2, 0x42, 0x58, 0xdf, 0x29, 0x1e, 0x78, 0x34, 0x45, 0x8f, 0xda, 0xb3, 0x94, 0xbd, 0xf8, 0x58, 0x34, 0x17, 
	0xff, 0x29, 0x1f, 0x79, 0x34, 0x38, 0xfd, 0xbb, 0x39, 0x3c, 0xe8, 0x79, 0x34, 0x26, 0xfa, 0xd2, 0x92, 0xaf, 0x69, 0x34, 0x37, 0x8e, 0xca, 0x39, 0x4a, 0xce, 0x87, 0x93, 0x42, 0x6f, 0xad, 0x29, 
	0x2a, 0xf7, 0xb1, 0x60, 0x4c, 0x14, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x7d, 0x14, 0x06, 0xa1, 0x62, 0x7f, 0xad, 0x29, 0x2a, 0xf6, 0x93, 0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x87, 
	0x93, 0x42, 0x6f, 0xad, 0x29, 0x3a, 0xf7, 0x46, 0x50, 0x48, 0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x24, 0x07, 0x67, 0x28, 0xfa, 0xd2, 0x92, 0xaf, 0x69, 0x34, 0x47, 0x8e, 0xca, 
	0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x26, 0xfa, 0xd2, 0x92, 0xaf, 0x69, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x24, 0x05, 0x65, 0x27, 0xfa, 0xe2, 0x90, 0xf7, 0x70, 0x69, 0x24, 
	0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 0x2f, 0x75, 0x56, 0x05, 0x92, 
	0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x87, 0x92, 0x40, 0x65, 0x72, 0x67, 0xff, 0x29, 0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 0x1f, 0x79, 
	0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x24, 0x06, 0x66, 0x17, 0xff, 0x29, 0x1f, 0x76, 0x60, 0x59, 0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 
	0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 0x1f, 0x8b, 0x17, 0x04, 0xc1, 0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x87, 0xd1, 0x40, 0x5a, 0x15, 
	0x17, 0xff, 0x29, 0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 0x1f, 0x76, 0x60, 0x59, 0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x24, 
	0x06, 0x66, 0x18, 0xff, 0x29, 0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xff, 0x29, 0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x79, 0x24, 
	0x16, 0x75, 0x61, 0x8f, 0xf2, 0x91, 0xf7, 0x66, 0x05, 0x92, 0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x87, 0x93, 0x41, 0x7f, 0xf2, 0x91, 0xf7, 0x93, 0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 
	0x87, 0x93, 0x41, 0x7f, 0xf2, 0x91, 0xf8, 0x67, 0x06, 0x92, 0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x87, 0x92, 0x40, 0x55, 0x62, 0x57, 0xff, 0x29, 0x1f, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 
	0x94, 0xac, 0xe8, 0x79, 0x34, 0x17, 0xef, 0x29, 0x1e, 0x79, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x7a, 0x14, 0x03, 0x23, 0x04, 0x14, 0x06, 0x67, 0x18, 0xef, 0x29, 0x1e, 0x77, 0x51, 
	0x43, 0x51, 0x13, 0x37, 0x44, 0x78, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x87, 0x64, 0x03, 0x91, 0x10, 0x36, 0x41, 0x7e, 0xf2, 0x91, 0xe7, 0x54, 0x00, 0x40, 0x01, 0x61, 0x04, 0x44, 0x47, 0x8e, 0xca, 
	0x93, 0x94, 0xac, 0xe8, 0x74, 0x40, 0x37, 0x10, 0x25, 0x20, 0x53, 0x51, 0x7e, 0xf2, 0x91, 0xe8, 0xb1, 0x71, 0x20, 0x61, 0x04, 0x34, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x73, 0x40, 0x3f, 
	0x11, 0x03, 0x34, 0x17, 0xef, 0x29, 0x1e, 0x73, 0x40, 0x18, 0x21, 0x04, 0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x73, 0x40, 0x1c, 0x11, 0x00, 0x20, 0x04, 0x24, 0x17, 0xef, 0x29, 0x1e, 
	0x87, 0x70, 0x1c, 0x11, 0x03, 0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x72, 0x40, 0x3d, 0x11, 0x02, 0x32, 0x05, 0x15, 0x18, 0xef, 0x29, 0x1e, 0x72, 0x40, 0x03, 0x00, 0x1c, 0x11, 0x03, 
	0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x72, 0x40, 0x39, 0x21, 0x03, 0x24, 0x17, 0xef, 0x29, 0x1e, 0x72, 0x40, 0x39, 0x21, 0x03, 0x24, 0x47, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x72, 
	0x40, 0x3b, 0x11, 0x10, 0x17, 0x71, 0x8e, 0xf2, 0x91, 0xe7, 0x25, 0x02, 0x32, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x68, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x86, 0x24, 0x03, 0xd1, 0x10, 0x02, 0x00, 0x32, 
	0x41, 0x7e, 0xf2, 0x91, 0xe7, 0x24, 0x03, 0x92, 0x10, 0x32, 0x44, 0x68, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x86, 0x24, 0x03, 0x92, 0x10, 0x32, 0x41, 0x7e, 0xf2, 0x91, 0xe8, 0x77, 0x11, 0x0b, 0x11, 
	0x03, 0x24, 0x46, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x62, 0x40, 0x3d, 0x11, 0x02, 0x32, 0x05, 0x15, 0x18, 0xef, 0x29, 0x1e, 0x72, 0x40, 0x33, 0x00, 0x1c, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xca, 
	0x93, 0x94, 0xac, 0xe8, 0x62, 0x40, 0x3d, 0x11, 0x00, 0x20, 0x03, 0x24, 0x17, 0xef, 0x29, 0x1e, 0x82, 0x50, 0x23, 0x20, 0x1c, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x62, 
	0x40, 0x3d, 0x11, 0x07, 0x67, 0x18, 0xef, 0x29, 0x1e, 0x82, 0x40, 0x39, 0x21, 0x03, 0x24, 0x46, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x62, 0x40, 0x39, 0x21, 0x03, 0x24, 0x17, 0xef, 0x29, 0x1e, 
	0x82, 0x40, 0x37, 0x00, 0x18, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x62, 0x40, 0x39, 0x11, 0x02, 0x72, 0x05, 0x15, 0x18, 0xef, 0x29, 0x1e, 0x8b, 0x17, 0x12, 0x07, 0x10, 
	0x32, 0x44, 0x68, 0xec, 0xa9, 0x39, 0x4a, 0xce, 0x86, 0x24, 0x03, 0x91, 0x10, 0x06, 0x00, 0x32, 0x41, 0x8e, 0xf2, 0x91, 0xe8, 0x24, 0x03, 0x92, 0x10, 0x32, 0x44, 0x68, 0xec, 0xa9, 0x39, 0x4a, 
	0xce, 0x86, 0x24, 0x03, 0x92, 0x10, 0x32, 0x41, 0x8e, 0xf2, 0x91, 0xe8, 0x77, 0x12, 0x0b, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xca, 0x93, 0x94, 0xac, 0xe8, 0x62, 0x40, 0x3d, 0x11, 0x02, 0x22, 0x05, 
	0x25, 0x18, 0xef, 0x29, 0x1e, 0x82, 0x40, 0x33, 0x00, 0x1c, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xda, 0x93, 0x94, 0xad, 0xe8, 0x62, 0x40, 0x3d, 0x11, 0x00, 0x20, 0x03, 0x24, 0x18, 0xef, 0x29, 0x5d, 
	0x85, 0x45, 0x33, 0x20, 0x1c, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xda, 0x93, 0x94, 0xad, 0xe8, 0x62, 0x40, 0x3b, 0x11, 0x10, 0x27, 0x71, 0x8d, 0xf2, 0x92, 0xd8, 0x51, 0x40, 0x39, 0x21, 0x03, 0x24, 
	0x46, 0x8e, 0xda, 0x93, 0x94, 0xad, 0xe8, 0x62, 0x40, 0x39, 0x21, 0x04, 0x24, 0x18, 0xdf, 0x29, 0x1d, 0x83, 0x40, 0x02, 0x00, 0x1c, 0x11, 0x03, 0x24, 0x46, 0x8e, 0xda, 0x93, 0x94, 0xad, 0xe8, 
	0x62, 0x40, 0x3d, 0x11, 0x02, 0x22, 0x05, 0x25, 0x18, 0xdf, 0x29, 0x1d, 0x87, 0x71, 0x20, 0xb1, 0x10, 0x32, 0x44, 0x68, 0xed, 0xa9, 0x39, 0x4a, 0xde, 0x86, 0x24, 0x03, 0x92, 0x10, 0x32, 0x41, 
	0x8d, 0xf2, 0x91, 0xd8, 0x34, 0x01, 0x82, 0x10, 0x32, 0x44, 0x68, 0xed, 0xa9, 0x39, 0x4a, 0xde, 0x86, 0x24, 0x03, 0xd1, 0x10, 0x02, 0x00, 0x42, 0x41, 0x8d, 0xf2, 0x91, 0xd8, 0x35, 0x02, 0x22, 
	0x01, 0xc1, 0x10, 0x32, 0x44, 0x68, 0xed, 0xa9, 0x39, 0x4a, 0xdf, 0x86, 0x24, 0x03, 0xd1, 0x10, 0x76, 0x71, 0x8d, 0xf2, 0x91, 0xd8, 0x34, 0x00, 0x20, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x68, 0xfd, 
	0xa9, 0x39, 0x4a, 0xdf, 0x86, 0x24, 0x03, 0x92, 0x10, 0x42, 0x41, 0x8d, 0xf2, 0x91, 0xd8, 0x34, 0x00, 0x60, 0x01, 0x81, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 
	0x91, 0x10, 0x26, 0x20, 0x51, 0x52, 0x68, 0xdf, 0x29, 0x1d, 0x8b, 0x17, 0x12, 0x07, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0x91, 0x10, 0x06, 0x00, 0x42, 0x41, 
	0x8d, 0xf2, 0x91, 0xd8, 0x34, 0x01, 0x82, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0x92, 0x10, 0x41, 0x42, 0x58, 0xdf, 0x29, 0x1d, 0x87, 0x71, 0x20, 0xb1, 0x10, 
	0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0xd1, 0x10, 0x22, 0x20, 0x52, 0x51, 0x8d, 0xf2, 0x91, 0xd8, 0x34, 0x00, 0x20, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 
	0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0xd1, 0x10, 0x02, 0x00, 0x42, 0x41, 0x8d, 0xf2, 0x95, 0xd8, 0x54, 0x53, 0x22, 0x01, 0xd1, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 
	0x03, 0xb1, 0x11, 0x02, 0x77, 0x18, 0xdf, 0x29, 0x2d, 0x85, 0x14, 0x03, 0x92, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0x92, 0x10, 0x41, 0x42, 0x58, 0xdf, 0x29, 
	0x2d, 0x85, 0x24, 0x00, 0x20, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0xd1, 0x10, 0x22, 0x20, 0x52, 0x51, 0x8d, 0xf2, 0x91, 0xd8, 0x77, 0x12, 0x0b, 
	0x11, 0x03, 0x24, 0x46, 0x8f, 0xdb, 0x93, 0x94, 0xbd, 0xf8, 0x62, 0x40, 0x39, 0x21, 0x03, 0x14, 0x25, 0x8d, 0xf2, 0x92, 0xd8, 0x52, 0x40, 0x18, 0x21, 0x03, 0x24, 0x46, 0x8f, 0xdb, 0x93, 0x94, 
	0xbd, 0xf8, 0x62, 0x40, 0x3d, 0x11, 0x00, 0x20, 0x04, 0x14, 0x25, 0x8d, 0xf2, 0x92, 0xd8, 0x62, 0x50, 0x22, 0x20, 0x1c, 0x11, 0x03, 0x24, 0x46, 0x8f, 0xdb, 0x93, 0x94, 0xbd, 0xf8, 0x62, 0x40, 
	0x3d, 0x11, 0x07, 0x67, 0x18, 0xdf, 0x29, 0x2c, 0x85, 0x24, 0x00, 0x20, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x68, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x86, 0x24, 0x03, 0x92, 0x10, 0x41, 0x42, 0x58, 0xcf, 
	0x29, 0x2c, 0x85, 0x24, 0x00, 0x70, 0x01, 0x71, 0x03, 0x24, 0x46, 0x8f, 0xdb, 0x93, 0x94, 0xbd, 0xf8, 0x52, 0x40, 0x37, 0x11, 0x02, 0xb1, 0x61, 0x8c, 0xf2, 0x92, 0xc8, 0x7a, 0x16, 0x12, 0x07, 
	0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0x81, 0x10, 0x07, 0x00, 0x41, 0x42, 0x58, 0xcf, 0x29, 0x2c, 0x85, 0x24, 0x01, 0x82, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 
	0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0x92, 0x10, 0x31, 0x42, 0x58, 0xcf, 0x29, 0x1c, 0x87, 0x71, 0x20, 0xb1, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0x92, 0x10, 
	0x31, 0x42, 0x58, 0xcf, 0x29, 0x2c, 0x85, 0x24, 0x01, 0x82, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0xd1, 0x10, 0x02, 0x00, 0x41, 0x42, 0x58, 0xcf, 0x29, 0x2c, 
	0x86, 0x25, 0x02, 0x22, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0xb1, 0x11, 0x02, 0x77, 0x18, 0xcf, 0x29, 0x2c, 0x85, 0x24, 0x01, 0x82, 0x10, 0x32, 
	0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0x92, 0x10, 0x41, 0x42, 0x58, 0xcf, 0x29, 0x2c, 0x85, 0x24, 0x00, 0x20, 0x01, 0xc1, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 
	0xdf, 0x85, 0x24, 0x03, 0xd1, 0x10, 0x22, 0x20, 0x51, 0x52, 0x68, 0xcf, 0x29, 0x1c, 0x87, 0x71, 0x20, 0xb1, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0xe1, 0x10, 
	0x01, 0x00, 0x41, 0x42, 0x58, 0xcf, 0x29, 0x2c, 0x85, 0x24, 0x01, 0x82, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0xd1, 0x10, 0x02, 0x00, 0x41, 0x42, 0x58, 0xcf, 
	0x29, 0x1c, 0x87, 0x60, 0x1c, 0x11, 0x03, 0x24, 0x45, 0x8f, 0xdb, 0x93, 0x94, 0xbd, 0xf8, 0x52, 0x40, 0x3b, 0x11, 0x10, 0x26, 0x62, 0x78, 0xcf, 0x29, 0x2c, 0x85, 0x24, 0x00, 0x20, 0x01, 0xc1, 
	0x10, 0x32, 0x44, 0x58, 0xfd, 0xb9, 0x39, 0x4b, 0xdf, 0x85, 0x24, 0x03, 0x92, 0x10, 0x41, 0x42, 0x58, 0xcf, 0x29, 0x2d, 0x85, 0x24, 0x01, 0x82, 0x10, 0x32, 0x44, 0x58, 0xfd, 0xb8, 0x39, 0x4a, 
	0xcf, 0x87, 0x34, 0x03, 0x92, 0x10, 0x42, 0x43, 0x78, 0xea, 0xa2, 0x93, 0xbf, 0x86, 0x44, 0x01, 0x82, 0x10, 0x34, 0x44, 0x68, 0xfc, 0xac, 0x29, 0x1a, 0xd1, 0x80, 0x55, 0x40, 0x39, 0x21, 0x04, 
	0x44, 0x05, 0x18, 0x0c, 0xe1, 0x93, 0xc8, 0x75, 0x54, 0x03, 0xa2, 0x10, 0x35, 0x44, 0x57, 0x8d, 0xb8, 0x29, 0x3b, 0xd8, 0x76, 0x40, 0x3d, 0x21, 0x04, 0x54, 0x27, 0x8b, 0xa1, 0x93, 0xaf, 0x85, 
	0x44, 0x03, 0x83, 0x10, 0x35, 0x43, 0x78, 0xdb, 0xc1, 0x91, 0xbd, 0x18, 0x04, 0x44, 0x03, 0xa3, 0x10, 0x34, 0x42, 0x58, 0xe8, 0x19, 0x2d, 0x86, 0x44, 0x03, 0xc3, 0x10, 0x34, 0x44, 0x58, 0xfc, 
	0xa8, 0x19, 0x4a, 0xce, 0x86, 0x44, 0x03, 0xe3, 0x10, 0x34, 0x42, 0x78, 0xb5, 0x92, 0xe8, 0x53, 0x40, 0x38, 0x41, 0x03, 0x44, 0x37, 0x8d, 0xb6, 0x94, 0xac, 0xf8, 0x54, 0x40, 0x19, 0x41, 0x03, 
	0x34, 0x26, 0x8b, 0x39, 0x2e, 0x85, 0x34, 0x01, 0xb4, 0x10, 0x43, 0x43, 0x78, 0xeb, 0x49, 0x4a, 0xcf, 0x85, 0x34, 0x03, 0xc4, 0x10, 0x33, 0x42, 0x78, 0xa1, 0x92, 0xc8, 0x53, 0x40, 0x1d, 0x41, 
	0x04, 0x34, 0x37, 0x8d, 0xb2, 0x94, 0xac, 0xe8, 0x63, 0x40, 0x3e, 0x41, 0x03, 0x34, 0x57, 0xe9, 0xa8, 0x63, 0x40, 0x1e, 0x41, 0x03, 0x34, 0x45, 0x8f, 0xdb, 0x19, 0x1b, 0xd1, 0x80, 0x43, 0x40, 
	0x1f, 0x41, 0x04, 0x24, 0x55, 0x8c, 0xd8, 0x52, 0x40, 0x38, 0x51, 0x03, 0x34, 0x37, 0x8e, 0xc1, 0xa3, 0xce, 0x86, 0x34, 0x03, 0x85, 0x10, 0x33, 0x43, 0x7e, 0xf7, 0x34, 0x01, 0x85, 0x10, 0x33, 
	0x49, 0x68, 0xfc, 0xab, 0xdf, 0x85, 0x34, 0x01, 0x95, 0x10, 0x42, 0x40, 0x71, 0x80, 0x63, 0x40, 0x19, 0x51, 0x04, 0x24, 0x35, 0x8f, 0xd1, 0xb3, 0xdf, 0x85, 0x34, 0x01, 0x95, 0x10, 0x42, 0x40, 
	0x61, 0x80, 0x63, 0x40, 0x19, 0x51, 0x04, 0x24, 0x35, 0x8f, 0xd1, 0xb3, 0xdf, 0x85, 0x34, 0x01, 0x95, 0x10, 0x42, 0x40, 0x61, 0x80, 0x63, 0x40, 0x18, 0x51, 0x03, 0x34, 0x35, 0x8f, 0xd1, 0xb3, 
	0xdf, 0x86, 0x34, 0x03, 0x85, 0x10, 0x33, 0x40, 0x61, 0x80, 0x73, 0x40, 0x38, 0x51, 0x03, 0x34, 0x96, 0x8f, 0xdb, 0xac, 0xe8, 0x74, 0x40, 0x1f, 0x41, 0x03, 0x34, 0x37, 0xed, 0x84, 0x40, 0x1f, 
	0x41, 0x04, 0x34, 0xa8, 0x8e, 0xca, 0x9c, 0xef, 0x85, 0x34, 0x03, 0xe4, 0x10, 0x33, 0x45, 0x58, 0xca, 0x86, 0x44, 0x01, 0xd4, 0x10, 0x43, 0x44, 0x68, 0xfd, 0xb1, 0x92, 0xbd, 0xe1, 0x80, 0x43, 
	0x40, 0x3c, 0x41, 0x03, 0x44, 0x18, 0xe1, 0x92, 0xc8, 0x54, 0x40, 0x1b, 0x41, 0x04, 0x34, 0x55, 0x8f, 0xec, 0xa2, 0x94, 0xbd, 0xf8, 0x74, 0x40, 0x3a, 0x41, 0x03, 0x44, 0x27, 0xfa, 0x29, 0x2d, 
	0x85, 0x44, 0x03, 0x84, 0x10, 0x34, 0x45, 0x58, 0xfe, 0xda, 0x39, 0x5a, 0xce, 0xf8, 0x75, 0x40, 0x1f, 0x31, 0x04, 0x44, 0x27, 0xfa, 0x49, 0x2c, 0x86, 0x54, 0x01, 0xd3, 0x10, 0x44, 0x45, 0x68, 
	0xfe, 0xdb, 0x59, 0x3a, 0xcd, 0xf1, 0x80, 0x55, 0x40, 0x1b, 0x31, 0x04, 0x44, 0x25, 0x8e, 0x79, 0x2b, 0xf8, 0x64, 0x03, 0x83, 0x10, 0x36, 0x45, 0x78, 0xfe, 0xca, 0x81, 0x95, 0xbd, 0xef, 0x86, 
	0x64, 0x03, 0xe2, 0x10, 0x36, 0x42, 0x78, 0xca, 0x19, 0x2d, 0x87, 0x74, 0x03, 0xa2, 0x10, 0x37, 0x40, 0x61, 0x83, 0xed, 0xca, 0xa1, 0x93, 0xac, 0xef, 0x18, 0x06, 0x81, 0x40, 0x3e, 0x11, 0x03, 
	0x74, 0x35, 0x78, 0xde, 0x19, 0x0d, 0x18, 0x05, 0x91, 0x41, 0x33, 0x61, 0x13, 0x3a, 0x14, 0x07, 0x18, 0x3f, 0xec, 0xae, 0x19, 0x1b, 0xc1, 0xe3, 0xf8, 0x75, 0xc3, 0x43, 0x68, 0xfc, 0xa2, 0x91, 
	0xbe, 0x18, 0x05, 0x93, 0x40, 0x61, 0x84, 0xfe, 0xdc, 0xba, 0x29, 0x4a, 0xcd, 0xef, 0x18, 0x17, 0x6d, 0x24, 0x56, 0x78, 0xfd, 0xbe, 0x29, 0x2a, 0xce, 0x28, 0x16, 0x5f, 0x14, 0x25, 0x67, 0x18, 
	0x0f, 0x1e, 0x2d, 0xba, 0xf2, 0x92, 0xbc, 0xd1, 0xe0, 0xf2, 0x81, 0x76, 0x15, 0x04, 0x54, 0x35, 0x56, 0x72, 0x84, 0xfe, 0xcb, 0xab, 0x39, 0x5a, 0xbc, 0xde, 0xf5, 0x80, 0x71, 0x70, 0x84, 0x81, 
	0xff, 0x1e, 0x3d, 0xcb, 0xad, 0x39, 0x1a, 0xb1, 0xc1, 0xdd, 0x1e, 0x1f, 0xf3, 0x80, 0xf2, 0xf3, 0xee, 0xdc, 0x1b, 0x0a, 0xc4, 0x90, 0xa1, 0xb0, 0xc1, 0xc0, 0xd6, 0xd1, 0xcc, 0x1b, 0x1a, 0xaf, 
	0x49, 0x1a, 0xa7, 0xb1, 0xaa, 0xf2, 0x90, 
}; // 2439 bytes

#endif