@echo off
setlocal ENABLEDELAYEDEXPANSION

rem DS203APP Win32 GCC support 
rem USER DEFINED VALUES
rem ===================================================
set CBASE=c:\gcc\bin\
set TARGET=DS203APP
rem ===================================================
 
set CROSS=!CBASE!arm-none-eabi-
set CC=!CROSS!gcc
set OBJCOPY=!CROSS!objcopy
set LD=!CROSS!ld
set AS=!CROSS!as
set STRIP=!CROSS!strip

if not exist !CC!.* (
  echo Compiler not found !
  goto :eof
)

set BINPATH=bin

set FWLIB_INC=..\FWLib\inc
set FWLIB_SRC=..\FWLib\src
set FWLIB_ASM=..\FWLib\asm

set APP_SRC=..\source
set APP_INC=..\include

set CFLAGS=-Wall -Os  -fno-common -mcpu=cortex-m3 -mthumb -msoft-float -fomit-frame-pointer -MD -I  !FWLIB_INC! -I !APP_INC! -I !APP_SRC!
set AFLAGS=-mcpu=cortex-m3 -mthumb
set LDFLAGS=-nostartfiles -mcpu=cortex-m3 -mthumb -march=armv7 -mfix-cortex-m3-ldrd -msoft-float --disable-newlib-supplied-syscalls  -Wl,-Map,DS203APP.map
set OBJS=Calibrat.o Draw.o Files.o Function.o ^
         Interrupt.o Main.o Menu.o Process.o Ident.o  startup.o ^
         stm32f10x_nvic.o  cortexm3_macro.o

set SRCS= !APP_SRC!\Calibrat.c !APP_SRC!\Draw.c !APP_SRC!\Files.c !APP_SRC!\Function.c !APP_SRC!\Interrupt.c ^
          !APP_SRC!\Main.c !APP_SRC!\Menu.c !APP_SRC!\Process.c !APP_SRC!\Ident.c  !APP_SRC!\startup.c ^
          !FWLIB_SRC!\stm32f10x_nvic.c 

if exist *.map del *.map
if exist *.elf del *.elf
if exist *.hex del *.hex
if exist *.bin del *.bin

echo Compiling...
if not exist !BINPATH! mkdir !BINPATH!
cd !BINPATH!
!CC! !AFLAGS! -c !FWLIB_ASM!\cortexm3_macro.s -o cortexm3_macro.o
!CC! !AFLAGS! -c !APP_SRC!\bios_gcc.S -o bios.o 
!CC! !CFLAGS! -c !SRCS!

echo Linking...
!CC!  -o !TARGET!.elf !LDFLAGS! -T ..\ds203app.lds !OBJS! 
!OBJCOPY! -O binary !TARGET!.elf !TARGET!.bin
!OBJCOPY! -O ihex !TARGET!.elf !TARGET!.hex

del *.d *.o 

if not exist !TARGET!.hex (
  echo Build failed
  goto :eof
) else (
  echo Build succeed !
)

