/******************** (C) COPYRIGHT 2009 e-Design Co.,Ltd. *********************
 File Name : Process.c       
 Version   : DS203_APP Ver 2.3x                                  Author : bure
*******************************************************************************/
#include "Interrupt.h"
#include "Function.h"
#include "Process.h"
#include "Draw.h"
#include "Menu.h"
#include "BIOS.h"
#include "File.h"

// FFT ////////////////////////////////////////////////////////////////////
short fr[FFTSize];		//s16
short fi[FFTSize];
u32 NFreq;
char NFreqStr[12];
int imax;			//s16
short PeakFreq;
char PeakFreqStr[12];
char FreqDivStr[12];
char FreqT1Str[12];
////////////////////////////////////////////////////////////////////////////

u32 Sum[4]={0,0,0,0};		//averaging accumulators
u16 NAvg[4]={0,0,0,0};
u16 VNAvg[2]={0,0};
s32 VxAvg[2]={0,0};
s32 VxSsq[2]={0,0};
u32 PxS[4]={0,0,0,0};
u32 TxS[4]={0,0,0,0};
u32 TxN[4]={0,0,0,0};

u16 TaS, TbS, TcS, TdS;            // cycles accumulated
u16 PaS, PbS, PcS, PdS;            // pulse width of the cumulative
u16 TaN, TbN, TcN, TdN;            // Cycle Count
s16  a_Mid_H, a_Mid_L;
s16  b_Mid_H, b_Mid_L;

s32 a_Avg, b_Avg, a_Ssq, b_Ssq;              // use signed integers, allows values at bottom of screen to be read
s16  a_Max, b_Max, a_Min, b_Min;              // 0 levels can get pushed up by calibration, bringing bottom of screen below 0
s16  aT_Max, bT_Max, aT_Min, bT_Min;         // for auto trigger, kept separate from meter data for curdeftime

s16 Posi_412,Posi_41_2, Posi_41, Posi_42, Posi_4_2, Posi_4F1, Posi_4F2, Posi_4F3, Posi_4F4;
s16 c_Max, d_Max, A_Posi, B_Posi;
u8  Full=1, BailoutFlag=0;
u16 JumpCnt,CountUnread;
u8  FrameMode;
u8  HoldOnNext =0;
u16 bag_max_buf = 4096;
u8  freerun=0;					//flags for auto trig mode
u8  exitflag=0;
u8  entryflag=0;
u8  ADCoffset=54; //shifts ADC/FIFO operating area away from non-linear first xx steps into previously unused linear 200-255 step area
u16 TempKp1=1024;
u8  CalFlag;
u8  CurDefTime=0;			
u8  Options;				//options flag
u8  HoldResetFlag=0;		
s16 Xtend;				//buffer size control
u8  discard=0;				//pre-signal trace blanking
s16 TrigSourceEnable=1;			//for alternate triggering, end of cycling, will go to chA
u32 ReverseBitMask;			
u32 BitMask;
u16 AltHoldoff=0;
s8  OldShift=0;
u8  ScalingOffset=40;			//centers Ka3 Y-position scaling factor on a step other than 0
                                        //25= same as calibration gain correction (Ka2) zero
                                        //40= same as y-position alignment offset (Ka1) zero calibration point  
u16 QStart[2]={0,0};			//quantization error start/stop reference positions	
u16 QEnd[2]={0,0};
u8  LastA_Mid=0;			//trigger level points for time meter quantization error calculations
u8  LastB_Mid=0;
u8  FirstA_Mid=0;
u8  FirstB_Mid=0;
u8  SumResetFlag=0;			//resets meter summing function
u32 UTmp;
u8  EnablePaS=1;			//controls update speed of PaS and PbS derived calcs (TH, TL, %duty) in large meters
u8  ClearHoldFlag=0;
u8  SlowModeSkip=0;
u8  ClearLeadingEdge=0;
u8  FFTflag=0;
u8  Normal=100;

void Beeper(u16 ms);					//alarms
void AlternateChannel(void);				//for A&B mode
void SetOffset(u8 channel,u8 range, s16 Ypos);		//sets Ka3,Kb3 zero level reference		
void BatLevelCompensation(void);
s8   InterpolateS8(u8 Ch, s8 L8,s8 H8);			//for battery level comp
u16  InterpolateU16(u8 Ch, u16 L16,u16 H16);
s16  QParam(u8 Ch, u16 Position,u8 service);		//for Qerror
s32  QError(u8 Ch, u16 Start, u16 End, u32 Utmp);	//quantization error compensation
void Average(u8 Ch);
void RunAvg(u8 Ch);
s32  VRunAvg(u8 Ch, s32 Value, s32 Sum);

uc16 Wait[27]=        {1000, 500, 200, 100, 65, 30, 20, 10, 5, 3,   			
                       2,      2,   2,   2,  2,  2,  2, 2, 2, 2,    			
                        2,      2,   2,   2,  2,  2,  2 };

uc16 shortwait[27]=        {300, 150, 100, 75, 35, 15, 10, 10, 5, 3,   	//for auto trig mode, set time to hold before going auto		
                        2,      2,   2,   2,  2,  2,  2, 2, 2, 2,    			
                        2,      2,   2,   2,  2,  2,  2 };

uc16 TrigDelayLoop[10]= {2,4,10,20,40,100,200,400,1000,2000};		//for trig holdoff, sets how many time loops before allowing rest of program to function

uc16 AltDelay[21]= {1000, 500, 300, 150, 75, 30, 15, 8, 4, 2};          //for alt trig mode, sets time to wait for trigger in other ch if untriggered

Y_attr *Y_Attr; 
X_attr *X_Attr; 
G_attr *G_Attr; 
T_attr *T_Attr;

u32 DataBuf[4096];
u8  TrackBuff  [X_SIZE * 4];         // curve track: i +0, i +1, i +2, i +3, respectively, placed one on the 4th track data

uc8 Exp[201]={    1,  2,  4,  6,  8, 10, 12, 14, 16, 18, 19, 21, 23, 25, 27, 29, 31, 33, 34, 36,     // 1+((n-X)/N) 
		 38, 40, 42, 43, 45, 47, 49, 50, 52, 54, 55, 57, 59, 61, 62, 64, 65, 67, 69, 70,     
		 72, 74, 75, 77, 78, 80, 81, 83, 84, 86, 87, 89, 90, 92, 93, 95, 96, 98, 99,101,
		102,103,105,106,107,109,110,111,113,114,115,117,118,119,120,122,123,124,125,127,
		128,129,130,131,133,134,135,136,137,138,139,140,141,143,144,145,146,147,148,149,
		150,151,152,153,154,155,156,156,157,158,159,160,161,162,163,164,164,165,166,167,
		168,168,169,170,171,171,172,173,174,174,175,176,176,177,178,178,179,180,180,181,
		182,182,183,183,184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191,
		191,192,192,192,193,193,194,194,194,194,195,195,195,196,196,196,196,197,197,197,
		197,197,198,198,198,198,198,198,198,198,199,199,199,199,199,199,199,199,199,199,
                200};


uc8 Hamming[512]=									
	 {12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 13, 13, 13, 13, 13, 13,
	 14, 14, 14, 14, 15, 15, 15, 15, 16, 16, 16, 16, 17, 17, 17, 18, 18, 19, 19, 19,
	 20, 20, 21, 21, 21, 22, 22, 23, 23, 24, 24, 25, 25, 26, 26, 27, 27, 28, 28, 29,
	 30, 30, 31, 31, 32, 32, 33, 34, 34, 35, 36, 36, 37, 38, 38, 39, 40, 40, 41, 42,
	 42, 43, 44, 45, 45, 46, 47, 47, 48, 49, 50, 51, 51, 52, 53, 54, 54, 55, 56, 57,
	 58, 58, 59, 60, 61, 62, 62, 63, 64, 65, 66, 67, 67, 68, 69, 70, 71, 72, 72, 73,
	 74, 75, 76, 77, 78, 78, 79, 80, 81, 82, 83, 84, 84, 85, 86, 87, 88, 89, 89, 90,
	 91, 92, 93, 94, 94, 95, 96, 97, 98, 99, 99,100,101,102,103,104,104,105,106,107,
	107,108,109,110,111,111,112,113,114,114,115,116,117,117,118,119,119,120,121,122,
	122,123,124,124,125,126,126,127,127,128,129,129,130,131,131,132,132,133,133,134,
	134,135,136,136,137,137,138,138,139,139,139,140,140,141,141,142,142,142,143,143,
	144,144,144,145,145,145,145,146,146,146,147,147,147,147,148,148,148,148,148,149,
	149,149,149,149,149,149,150,150,150,150,150,150,150,150,150,150,150,150,150,150,
	150,150,150,150,150,150,149,149,149,149,149,149,149,148,148,148,148,148,147,147,
	147,147,146,146,146,145,145,145,145,144,144,144,143,143,142,142,142,141,141,140,
	140,139,139,139,138,138,137,137,136,136,135,134,134,133,133,132,132,131,131,130,
	129,129,128,127,127,126,126,125,124,124,123,122,122,121,120,119,119,118,117,117,
	116,115,114,114,113,112,111,111,110,109,108,107,107,106,105,104,104,103,102,101,
	100, 99, 99, 98, 97, 96, 95, 94, 94, 93, 92, 91, 90, 89, 89, 88, 87, 86, 85, 84,
	 84, 83, 82, 81, 80, 79, 78, 78, 77, 76, 75, 74, 73, 72, 72, 71, 70, 69, 68, 67,
	 67, 66, 65, 64, 63, 62, 62, 61, 60, 59, 58, 58, 57, 56, 55, 54, 54, 53, 52, 51,
	 51, 50, 49, 48, 47, 47, 46, 45, 45, 44, 43, 42, 42, 41, 40, 40, 39, 38, 38, 37,
	 36, 36, 35, 34, 34, 33, 32, 32, 31, 31, 30, 30, 29, 28, 28, 27, 27, 26, 26, 25,	
	 25, 24, 24, 23, 23, 22, 22, 21, 21, 21, 20, 20, 19, 19, 19, 18, 18, 17, 17, 17,
	 16, 16, 16, 16, 15, 15, 15, 15, 14, 14, 14, 14, 13, 13, 13, 13, 13, 13, 12, 12,
	 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12};

s8  Ka1[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; // A channel low error correction coefficient
s8  Kb1[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; // B channel low error correction coefficient
u16 Ka2[10] ={1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024}; // A channel gain error correction coefficient
u16 Kb2[10] ={1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024}; // B channel gain error correction factor
s8  Ka3[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; // A channel high error correction coefficient
s8  Kb3[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; // B-channel high bit error correction coefficient

s8  HKa1[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; //High battery voltage correction factors (eg: AC charger connected)
s8  HKb1[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; 
u16 HKa2[10] ={1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024}; 
u16 HKb2[10] ={1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024};
s8  HKa3[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; 
s8  HKb3[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; 

s8  LKa1[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; //Low battery voltage correction factors  (~1/4 charge)
s8  LKb1[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; 
u16 LKa2[10] ={1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024}; 
u16 LKb2[10] ={1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024};
s8  LKa3[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0}; 
s8  LKb3[10] ={   0,    0,    0,    0,    0,    0,    0,    0,    0,    0};

u16 HiBatLevel[2]={0,0};		//high battery correction reference level
u16 LoBatLevel[2]={0,0};               //low battery correction reference level
u16 PrevBatLevel=0;		
u16 VDiff[2]={0,0};

D_tab D_Tab[23] ={  // pulse waveform output driver table, based on the 72MHz frequency
//    STR      PSC     ARR       DUTY% 
  {" !1Hz! ", 1800-1,  40000-1,     50},
  {" !2Hz! ", 1800-1,  20000-1,     50},
  {" !5Hz! ", 1800-1,   8000-1,     50},
  {" 10Hz ",  180-1,  40000-1,     50},
  {" 20Hz ",  180-1,  20000-1,     50},
  {" 50Hz ",  180-1,   8000-1,     50},
  {"!100Hz!", 180-1,   4000-1,     50},
  {"!200Hz!", 180-1,   2000-1,     50},
  {"!500Hz!", 180-1,    800-1,     50},
  {" 1KHz ",  180-1,    400-1,     50},
  {" 2KHz ",  180-1,    200-1,     50},
  {" 5KHz ",   18-1,    800-1,     50},
  {"!10KHz!",  18-1,    400-1,     50},
//{"!20KHz!",  18-1,    200-1,     50},   // use 25Khz instead so display matches the analog version
  {"!25KHz!",   1-1,   2880-1,     50},
  {"!50KHz!",   1-1,   1440-1,     50},
  {"100KHz",    1-1,    720-1,     50},
  {"200KHz",    1-1,    360-1,     50},
  {"500KHz",    1-1,    144-1,     50},
  {" 1MHz ",    1-1,     72-1,     50},
  {" 2MHz ",    1-1,     36-1,     50},
  {" 4MHz ",    1-1,     18-1,     50},
  {" 6MHz ",    1-1,     12-1,     50},
  {" 8MHz ",    1-1,      9-1,     50}};

A_tab A_Tab[15] ={ // analog waveform output driver table synthesis, based on the 72MHz frequency, per 36



  {"! 1Hz !", 10-1,  50000-1},	// modified to show proper freq without use of special SYS version
  {"! 2Hz !", 10-1,  25000-1},
  {"! 5Hz !", 10-1,  10000-1},
  {" 10Hz ",  10-1,   5000-1},
  {" 20Hz ",  10-1,   2500-1},
  {" 50Hz ",  10-1,   1000-1},
  {"!100Hz!", 10-1,    500-1},
  {"!200Hz!", 10-1,    250-1},
  {"!500Hz!", 10-1,    100-1},
  {" 1KHz ",  10-1,     50-1},
  {" 2KHz ",  10-1,     25-1},
  {" 5KHz ",  10-1,     10-1},
  {"!10KHz!", 10-1,      5-1},
  {"!25KHz!", 10-1,      2-1},	//could not get 20Khz without special SYS version, using 25Khz instead
  {"!40KHz!", 10-1,      2-1}};	//not used, stops at 25Khz


u16   ATT_DATA[72];

uc16 SIN_DATA[72] = 
{2048,2226,2404,2578,2748,2914,3072,3223,3364,3496,3617,3726,3822,
3904,3972,4026,4065,4088,4095,4088,4065,4026,3972,3904,3822,3726,3617,
3496,3364,3223,3072,2914,2748,2578,2404,2226,2048,1870,1692,1518,1348,
1182,1024,873,732,600,479,370,274,192,124,70,31,8,0,8,31,70,124,192,274,
370,479,600,732,873,1024,1182,1348,1518,1692,1870,};

uc16 TRG_DATA[72] =  // triangle wave data
  {0,114,228,342,456,570,684,798,912,1026,1140,1254,1368,1482,1596,1710,1824,1938,2052,
2166,2280,2394,2508,2622,2736,2850,2964,3078,3192,3306,3420,3534,3648,3762,3876,3990,4095,
3990,3876,3762,3648,3534,3420,3306,3192,3078,2964,2850,2736,2622,2508,2394,2280,2166,2052,
1938,1824,1710,1596,1482,1368,1254,1140,1026,912,798,684,570,456,342,228,114};

uc16 SAW_DATA[72] =  // Sawtooth wave data                                                                                             //         
  {0,
57,112,168,224,280,336,392,448,504,560,616,672,728,784,840,896,952,1008,1064,1120,1176,1232,
1288,1344,1400,1456,1512,1568,1624,1680,1736,1792,1848,1904,1960,2016,2072,2128,2184,2240,2296,
2352,2408,2464,2520,2576,2632,2688,2744,2800,2856,2912,2968,3024,3080,3136,3192,3248,3304,3360,
3416,3472,3528,3584,3640,3696,3752,3808,3864,3920,3976};   // 360   

uc16 DIGI_DATA[2] =  // Square wave data                                                                                             //         
  {0,4095};    

void BackGround_Reset(void)
{ 
  u16 i, j;
  u8 TitleLimit=13;
  if ((FlagMeter==2)&&(UpdateMeter!=4))TitleLimit=11;
  
  __Clear_Screen(BLACK);       
  for(i=0; i<TitleLimit; i++) for(j=0; j<4; j++) Title[i][j].Flag |= UPDAT;
  for(i=0; i<9; i++)                     Meter[i].Flag    |= UPDAT;
  if(Current != FILE) {
    Title[FILE][0].Flag &= !UPDAT;
    Title[FILE][1].Flag &= !UPDAT;
    Title[FILE][3].Flag &= !UPDAT;
  }  
   Update = 1;                  // return back the jumper settings
}

/*******************************************************************************
 App_init: Displays the window waveform data initialization
*******************************************************************************/
void App_init(void)
{ 
  __Set(ADC_CTRL, EN );       
  __Set(STANDBY, DN);          // exit the power saving state
  Delayms(20); 
  __Set(FIFO_CLR, W_PTR); 
  BackGround_Reset();
  PD_Cnt = 600;
}

/*******************************************************************************
 _init: Displays the window waveform data initialization <<<< THIS (AND REF_WAVE) COULD BE ELIMINATED? DOES NOT SEEM TO BE USED
*******************************************************************************/
/*void View_init(void)
{ 
  u16 i, j = 0, k = 0;
  for(i = 0; i < X_SIZE * 4; i += 4){
    TrackBuff[i] = 160;
    TrackBuff[i+1] = Get_Ref_Wave(i/4);
    if((i%160)==0) j = 1 - j;
    TrackBuff[i+2] = 60 + (j * 17);
    if((i%64)==0) k = 1 - k;
    TrackBuff[i+3] = 20 + (k * 17);
  }
}*/
/*******************************************************************************
 Update_Range: 
*******************************************************************************/
void Update_Range(void) 
{
  __Set(ADC_CTRL, EN);       
  __Set(ADC_MODE, SEPARATE);                        // Set Separate mode ((Range + 1)*25)

  __Set(CH_A_COUPLE, Title[TRACK1][COUPLE].Value);
  __Set(CH_A_RANGE,  Title[TRACK1][RANGE].Value);
  SetOffset(0,_A_Range,_1_posi);
  __Set(CH_B_COUPLE, Title[TRACK2][COUPLE].Value);
  __Set(CH_B_RANGE,  Title[TRACK2][RANGE].Value);
  SetOffset(1,_B_Range,_2_posi);
}
/*******************************************************************************
 Update_Base: 
*******************************************************************************/
void Update_Base(void) 
{
  u16 i;
  
  __Set(ADC_CTRL, EN);       
  i = Title[T_BASE][BASE].Value;     
  __Set(T_BASE_PSC, X_Attr[i].PSC);
  __Set(T_BASE_ARR, X_Attr[i].ARR);
  Wait_Cnt = Wait[_T_base];
  if(_Status == RUN) __Set(FIFO_CLR, W_PTR);      // FIFO write pointer reset
}
/*******************************************************************************
 Update_Output: 
*******************************************************************************/
void Update_Output(void) 
{
  
  u8 att;

  if(_Kind == SINE){
    for(att=0; att <72; att++){ 
     //ATT_DATA[att]=(SIN_DATA[att]*Title[OUTPUT][OUTATT].Value)/100;}	                       //zero axis at 0, causes distortion as level is dropped
     ATT_DATA[att]=(((Title[OUTPUT][OUTATT].Value*(SIN_DATA[att]-2048))/100)+2048);}           //establish zero axis at mid of full output level
     DMA2_Channel4->CCR &= ~DMA_CCR1_EN;
      //__Set(ANALOG_PSC,  A_Tab[_Frqn].PSC);								     //used with special sys version	
      __Set(ANALOG_CNT, 72);
      __Set(ANALOG_PTR, (u32)ATT_DATA);
    DMA2_Channel4->CCR |= DMA_CCR1_EN;
      __Set(ANALOG_ARR, A_Tab[_Frqn].ARR);
    
  }
  if(_Kind == SAW){
   for(att=0; att <72; att++){ 
     //ATT_DATA[att]=(SAW_DATA[att]*Title[OUTPUT][OUTATT].Value)/100;}
     ATT_DATA[att]=(((Title[OUTPUT][OUTATT].Value*(SAW_DATA[att]-2048))/100)+2048);}           
     DMA2_Channel4->CCR &= ~DMA_CCR1_EN;
      //__Set(ANALOG_PSC,  A_Tab[_Frqn].PSC);
      __Set(ANALOG_CNT, 72);
      __Set(ANALOG_PTR, (u32)ATT_DATA);
     DMA2_Channel4->CCR |= DMA_CCR1_EN;
      __Set(ANALOG_ARR, A_Tab[_Frqn].ARR);
  }
  if(_Kind == TRIANG){
    for(att=0; att <72; att++){ 
     //ATT_DATA[att]=(TRG_DATA[att]*Title[OUTPUT][OUTATT].Value)/100;}
     ATT_DATA[att]=(((Title[OUTPUT][OUTATT].Value*(TRG_DATA[att]-2048))/100)+2048);}          
     DMA2_Channel4->CCR &= ~DMA_CCR1_EN;
      //__Set(ANALOG_PSC,  A_Tab[_Frqn].PSC);
      __Set(ANALOG_CNT, 72);
      __Set(ANALOG_PTR, (u32)ATT_DATA);
    DMA2_Channel4->CCR |= DMA_CCR1_EN;
      __Set(ANALOG_ARR, A_Tab[_Frqn].ARR);
  }
  
  if(_Kind == DIGI){
    for(att=0; att <72; att++){ 
       ATT_DATA[att]=(((Title[OUTPUT][OUTATT].Value*(DIGI_DATA[(att/36)]-2048))/100)+2048);
    }           

     DMA2_Channel4->CCR &= ~DMA_CCR1_EN;
      //__Set(ANALOG_PSC,  A_Tab[_Frqn].PSC);
      __Set(ANALOG_CNT, 72);
      __Set(ANALOG_PTR, (u32)ATT_DATA);
    DMA2_Channel4->CCR |= DMA_CCR1_EN;
      __Set(ANALOG_ARR, A_Tab[_Frqn].ARR);
  }

  if(_Kind == PWM){
    __Set(DIGTAL_PSC, D_Tab[_Frqn].PSC);
    __Set(DIGTAL_ARR, D_Tab[_Frqn].ARR);
    __Set(DIGTAL_CCR, ((D_Tab[_Frqn].ARR+1)*(100-Title[OUTPUT][DUTYPWM].Value))/100);
  }
}
/*******************************************************************************
 Update_Trig: 
*******************************************************************************/
void Update_Trig(u8 service) 
{ 

  if(_T1 > _T2)  __Set(T_THRESHOLD, ((((TempKp1*_T1)+512)/1024)-(((TempKp1*_T2)+512)/1024)));    //compensate for interpolated ranges
  else           __Set(T_THRESHOLD, ((((TempKp1*_T2)+512)/1024)-(((TempKp1*_T1)+512)/1024))); 	 
                                                                                                 

 if(service){
  __Set(TRIGG_MODE,  (_Tr_source<< 3)+_Tr_kind);
  if(Title[TRIGG][SOURCE].Value == TRACK1){ 
  __Set(V_THRESHOLD, ((((_Vt1-Ka1[_A_Range]-_1_posi)*1024)/Ka2[_A_Range])&0xFF)+ADCoffset+_1_posi); 
  }
  if(Title[TRIGG][SOURCE].Value == TRACK2){ 
  __Set(V_THRESHOLD, ((((_Vt2-Kb1[_B_Range]-_2_posi)*1024)/Kb2[_B_Range])&0xFF)+ADCoffset+_2_posi); 
  }

 }else{
  __Set(TRIGG_MODE,  (TrigSourceEnable<< 3)+_Tr_kind);
  if(TrigSourceEnable == TRACK1){ 
  __Set(V_THRESHOLD, ((((_Vt1-Ka1[_A_Range]-_1_posi)*1024)/Ka2[_A_Range])&0xFF)+ADCoffset+_1_posi); 
  }
  if(TrigSourceEnable == TRACK2){ 
  __Set(V_THRESHOLD, ((((_Vt2-Kb1[_B_Range]-_2_posi)*1024)/Kb2[_B_Range])&0xFF)+ADCoffset+_2_posi); 
  }

 }

}
/*******************************************************************************
 Process: Calculate processing buffer data
*******************************************************************************/
void Process(void)
{ 
  s16 i, j = 0, k, V[8] = {0},a,b, n = 0;
  s32 Tmp;
  u8  Ch[4], C_D=0;
  s8  Sa = 2, Sb = 2, Sc = 2, Sd = 2;     // time status
  u16 Ta, Tb, Tc, Td;                     // pulse width count
  u16 Pa, Pb, Pc, Pd;			  // holds + side transition of wave for PaS
  u8  jj;
  s16 h;
  s16 start;                              //for cursor defined meters
  u16 finish;
  s8  Shift=0;				  //jitter compensation value
  u8  TriggerPosition=150;
  u8  DTriggerPosition=150;
  u8  TimeCalcHyst=2;			  //time based calculations trigger point hysteresis value
  s16  AWavCenter;                          
  s16  BWavCenter;
  s16  X,Y;
  u16  LeftSkirt=0;

  Ta = Tb = Tc = Td = 0;
  Pa = Pb = Pc = Pd = 0;

  TaS = 0; TbS = 0; TcS = 0; TdS = 0; 
  PaS = 0; PbS = 0; PcS = 0; PdS = 0; 
  TaN = 0; 
  TbN = 0; TcN = 0; TdN = 0; 

  Posi_412 = _4_posi - _1_posi - _2_posi;
  Posi_41_2 = _4_posi - _1_posi + _2_posi;
  Posi_41  = _4_posi - _1_posi;
  Posi_42  = _4_posi - _2_posi;
  Posi_4_2 = _4_posi + _2_posi;
  Posi_4F1 = _4_posi - FileBuff[ 399];
  Posi_4F2 = _4_posi - FileBuff[ 799];
  Posi_4F3 = _4_posi - FileBuff[ 1199];
  Posi_4F4 = _4_posi - FileBuff[1599];

  A_Posi   = _1_posi-Ka1[_A_Range];    
  B_Posi   = _2_posi-Kb1[_B_Range];    
  
  //a_Max = A_Posi; b_Max = B_Posi;      	 
  //a_Min = A_Posi; b_Min = B_Posi;             
  a_Max=-0x7FFF;a_Min=0x7FFF;          //in case max is negative or min is positive (as with DC voltages)	
  b_Max=-0x7FFF;b_Min=0x7FFF;          //originally would initiate @i=0 but with init @i=4 could transfer in slow mode before init

  aT_Max = A_Posi; bT_Max = B_Posi;    // Used for calculating auto trigger
  aT_Min = A_Posi; bT_Min = B_Posi;             

  if((_3_posi + 20)>= Y_BASE+Y_SIZE)  c_Max = Y_BASE+Y_SIZE-1;
  else                                c_Max = _3_posi + 20;
  if((_4_posi + 20)>= Y_BASE+Y_SIZE)  d_Max = Y_BASE+Y_SIZE-1;
  else                                d_Max = _4_posi + 20;

     //if ((((_Mode==NORH)||(_Mode==NORC))&&(_T_base < 10)&&(Options&1))||(ShowFFT==1)){
     if ((((_Mode==NORH)||(_Mode==NORC))&&(_T_base < 10)&&(Options&1))||(_4_source>9)){
       Xtend=150;							//stabilize buffer size in holdoff mode to cover pre-trigger section
     }else{								//optimize size for best aquisition
       Xtend=_X_posi.Value;
       if (FlagMeter>0) Xtend-=86;
     } 

      if(Title[7][1].Value>3)TriggerPosition=149;else TriggerPosition=150;		//time based triggers occur 1 sample earlier

      switch (_T_base){									//digital ch trig positions for interpolated ranges
        case 17:
	  DTriggerPosition=149;
          break;
        case 18:
	  DTriggerPosition=148;
          break;
        case 19:
          if(Title[7][1].Value>3)DTriggerPosition=147;else DTriggerPosition=148;		
          break;
        case 20:
          if(Title[7][1].Value>3)DTriggerPosition=146;else DTriggerPosition=147;		
          break;
        case 21:
          if(Title[7][1].Value>3)DTriggerPosition=145;else DTriggerPosition=146;		
      }

    if(_Status==RUN)TempKp1=_Kp1;	                   // eliminates improper change of wave timebase by Kp1 while in hold mode at faster timebases
    k =((1024 - TempKp1)*TriggerPosition + 512)/1024 + _X_posi.Value;  // window position in the calculation of the interpolation of the correction value

    start=(((((TempKp1*1000)/1024)*Title[T_VERNIE][0].Value)+500)/1000)+k-((1024/TempKp1)/4);	//calculate start/stop positions for
    if((_T_base==20)&&(Title[7][0].Value<2))start--; 						//cursor defined meters
    if(_T_base==19)start--; 
    if((_T_base==18)&&(Title[7][1].Value>3))start--; 
    finish=(((((TempKp1*1000)/1024)*Title[T_VERNIE][1].Value)+500)/1000)+k;    

    if ((Title[TRIGG][SOURCE].Value>1)&&(Title[TRIGG][SOURCE].Value<4)){start+=5;finish+=5;}      // adjust to align with shifted wave when triggering from dig ch

    bag_max_buf = get_bag_max_buf();

    if(Title[T_VERNIE][T1].Value==0)start=0;			// lock out time cursors from restricting measurements if
    if(FlagMeter){                                              // at extreme right or left in cursor defined meter mode
      if(Title[T_VERNIE][T2].Value==300)finish=bag_max_buf;
    }else{
      if(Title[T_VERNIE][T2].Value==386)finish=bag_max_buf;
    }

    if (CurDefTime==0){			
      NSamples=(bag_max_buf-4);			//count starts @ i=4 to eliminate noise on some ranges			
      a_Avg=NSamples/2;
      b_Avg=NSamples/2;
    }else{
       if (finish>start){
         NSamples=(finish-start);
         if (NSamples>1){
	    NSamples--;
            a_Avg=NSamples/2;
            b_Avg=NSamples/2;
         }else{
          a_Avg=A_Posi;
          b_Avg=B_Posi;
         }
       }else{
        NSamples=1;
        a_Avg=A_Posi;
        b_Avg=B_Posi;
       }
    }  
    a_Ssq=b_Ssq=NSamples/2;
    if((_4_source==15)||(_4_source==13))Normal=100; else Normal=85;                //normalize FFT levels

    if ((exitflag==1)&&(JumpCnt!=0)) exitflag=0;      // leave exitflag on at beginning of frame so if complete frame is done, allow freerun 

    for(i=0; i <bag_max_buf; i++){
      if(((_T_base > 10)||(freerun==1)||((_Mode==X_Y)&&(_T_base>9)))&&(_Status == RUN)){		//>500uS/div
           if(Title[TRIGG][SOURCE].Value==4) TransferFIFO(i,1); else TransferFIFO(i,0);   
      }
      else if((__Get(FIFO_EMPTY)==0)&&(i == JumpCnt)&&(_Status == RUN))     
  	{
            JumpCnt++;
            if(Title[TRIGG][SOURCE].Value==4) TransferFIFO(i,1); else TransferFIFO(i,0);
  	}

      Ch[A] = (DataBuf[i] & 0xFF );              	      // now only used for time measurements	
	V[A]=(Ch[A]-ADCoffset);				      // for wave trace,(also now used for meters, with signed vars) load into signed var and no clipping
	if (Ch[A]<ADCoffset) Ch[A]=ADCoffset;       	      // clip at new 0 level, needed for unsigned var	    	
	Ch[A]-=ADCoffset;				      // with ypos shift canceling this, moves operating point back to normal	

      Ch[B] = ((DataBuf[i] >> 8) & 0xFF);
	V[B]=(Ch[B]-ADCoffset);
	if (Ch[B]<ADCoffset) Ch[B]=ADCoffset;               	    	
	Ch[B]-=ADCoffset;

	  if(i == 4) {								// used for auto trigger
	      aT_Max = V[A];	// statistics channel A maximum
		aT_Min = aT_Max;	// statistics channel A minimum
		bT_Max = V[B];	// statistics channel B maximum
		bT_Min = bT_Max;	// statistics channel B minimum
	  }else{         				  // not the first values
  		if(V[A] > aT_Max)  aT_Max = V[A];         // statistics channel A maximum
		if(V[A] < aT_Min)  aT_Min = V[A];         // statistics channel A minimum
		if(V[B] < bT_Min)  bT_Min = V[B];         // statistics channel B minimum
		if(V[B] > bT_Max)  bT_Max = V[B];         // statistics channel B maximum
	  }

		// FFT ///////////////////////////////////////

      	    if (i < FFTSize){
		if ((_4_source==SPEC_A)|| (_4_source==15)||(_4_source==14))
                    fr[i]=(((Ch[A]*Hamming[i])+(Hamming[i]>>1))/Normal)<<2;

		if ((_4_source==SPEC_B)|| (_4_source==13)||(_4_source==12))
                    fr[i]=(((Ch[B]*Hamming[i])+(Hamming[i]>>1))/Normal)<<2;
            }

		////////////////////////////////////// FFT ///


      if (i>5) {C_D = DataBuf[i-6] >>16;}else C_D=0;	  // align digital chs with analogs

//if(FlagMeter>0){	//could be used to speed up display refresh rate when meters are turned off?
      if (((CurDefTime==0)&&(i>3)&&(i<bag_max_buf))||((i>start)&&(i<finish))){		//for voltage measurements, only for analog channels

        a_Avg += V[A];                                    // use signed vars, otherwise, values at bottom of screen, pushed up by Kx1, now below zero, fail to register
        Tmp = V[A]- A_Posi;
        a_Ssq +=(Tmp * Tmp);                              // statistical sum of squares of the A channel

        b_Avg += V[B];                                    // cumulative average channel B, DC
        Tmp = V[B]- B_Posi;
        b_Ssq +=(Tmp * Tmp);                              // statistical sum of squares of the B channel

	  if((i == 4)||((CurDefTime>0)&&(i==(start+1)))) {		  // read first values - max = min = values
	        a_Max = V[A];	// statistics channel A maximum
		a_Min = a_Max;	// statistics channel A minimum
		b_Max = V[B];	// statistics channel B maximum
		b_Min = b_Max;	// statistics channel B minimum
	  }else{			                // not the first values
  		if(V[A] > a_Max)  a_Max = V[A];         // statistics channel A maximum
		if(V[A] < a_Min)  a_Min = V[A];         // statistics channel A minimum
		if(V[B] < b_Min)  b_Min = V[B];         // statistics channel B minimum
		if(V[B] > b_Max)  b_Max = V[B];         // statistics channel B maximum
	  }
      }

     if ((CurDefTime==0)||((i>start-1)&&(i<(finish+1)))){	//for time related measurements for both dig and analog channels

      if((i>3)&&(i<(bag_max_buf-3))){ 

       if((CurDefTime==0)||(i<finish)){

        if(Ch[A] > a_Mid_H){
          if (Sa<2){					//don't initiate if value is above trigger point when starting out
            if(Sa == 0){
		  TaS = i;                    					
                  LastA_Mid=a_Mid_H;			//save relevant time meter trigger level for quantization compensation
              if(Ta==0){				//use Ta to initiate
                  Ta=i;					//first + crossing point saved 
                  QStart[0]=i;				//start position saved for quantization calcs 
                  FirstA_Mid=a_Mid_H;
		  }else{				//after initialized, count + crossings, add + wave sampling count to PaS
		    TaN++;                              //increment TaN only after initial crossing point saved
		    PaS += Pa;				//only add positive part if whole wave is considered	
              }
		}
            Sa = 1;  
          } 
        } else { 
          if(Ch[A] < a_Mid_L){
            if(Sa == 2){
		  Sa=0; 				//initiate only after going below trigger point so partial wave does not get counted
		  PaS=0;				//initiate PaS
            }
	      if(Sa == 1){
                  Sa = 0;
		  Pa = i-TaS;				//save for PaS
            }
          }
        }

       if(Ch[B] > b_Mid_H){
          if (Sb<2){					
            if(Sb == 0){
		  TbS = i;                    					
                  LastB_Mid=b_Mid_H;			//save relevant time meter trigger level for quantization compensation
              if(Tb==0){				
                  Tb=i;					 
                  QStart[1]=i;				//start position saved for quantization calcs 
                  FirstB_Mid=b_Mid_H;
		  }else{
		    TbN++;                         
		    PbS += Pb;					
              }
		}
            Sb = 1;  
          } 
        } else { 
          if(Ch[B] < b_Mid_L){
            if(Sb == 2){
		  Sb=0; 					
		  PbS=0;					
            }
	      if(Sb == 1){
              Sb = 0;
		  Pb = i-TbS;				
            }
          }
        }
       }  

    if((i>10)&&((CurDefTime==0)||(i>start))){                        //eliminate noise at start of wave causing errors in dig channels

       if(C_D & 1){
          if (Sc<2){					
            if(Sc == 0){
		  TcS = i;                    					
              if(Tc==0){				
                Tc=i;					 
		  }else{
		    TcN++;                         
		    PcS += Pc;					
              }
		}
            Sc = 1;  
          } 
        } else { 
            if(Sc == 2){
		  Sc=0; 					
		  PcS=0;					
            }
	      if(Sc == 1){
              Sc = 0;
		  Pc = i-TcS;				
            }
	  } 

       if(C_D & 2){
          if (Sd<2){					
            if(Sd == 0){
		  TdS = i;                    					
              if(Td==0){				
                Td=i;					 
		  }else{
		    TdN++;                         
		    PdS += Pd;					
              }
		}
            Sd = 1;  
          } 
        } else { 
            if(Sd == 2){
		  Sd=0; 					
		  PdS=0;					
            }
	      if(Sd == 1){
              Sd = 0;
		  Pd = i-TdS;				
            }
        }

      }	//if i>8
    } //if i>3
  }  //if curdeftime
//}    //if flagmeter>0
                                                                                                                //calculate jitter
      if((i==TriggerPosition)&&(_T_base > 16)&&(_Mode!=SCAN)&&(__Get(FIFO_START)!=0)){				//analog channels
        if((Title[7][1].Value==1)||(Title[7][1].Value==3)||(Title[7][1].Value==4)||(Title[7][1].Value==5)){	//positive transitions
          if ((Title[7][0].Value==0)||((Title[TRIGG][SOURCE].Value==4)&&(TrigSourceEnable==0))){										//select trigger channel
            b=(Ka1[_A_Range]+A_Posi+(Ka2[_A_Range]*(V[A]-A_Posi)+ 512)/1024)-_Vt1;
	    a=_Vt1-(Ka1[_A_Range]+A_Posi+(Ka2[_A_Range]*(((DataBuf[TriggerPosition-1]& 0xFF)-ADCoffset)-A_Posi)+ 512)/1024);        
          }else if((Title[7][0].Value==1)||((Title[TRIGG][SOURCE].Value==4)&&(TrigSourceEnable==1))){
            b=(Kb1[_B_Range]+B_Posi+(Kb2[_B_Range]*(V[B]-B_Posi)+ 512)/1024)-_Vt2;
	    a=_Vt2-(Kb1[_B_Range]+B_Posi+(Kb2[_B_Range]*((((DataBuf[TriggerPosition-1]>>8)&0xFF)-ADCoffset)-B_Posi)+ 512)/1024);        
          }else goto bypasscalculate;
        }else{													//negative transitions
          if ((Title[7][0].Value==0)||((Title[TRIGG][SOURCE].Value==4)&&(TrigSourceEnable==0))){										
            b=_Vt1-(Ka1[_A_Range]+A_Posi+(Ka2[_A_Range]*(V[A]-A_Posi)+ 512)/1024);
	    a=(Ka1[_A_Range]+A_Posi+(Ka2[_A_Range]*(((DataBuf[TriggerPosition-1]& 0xFF)-ADCoffset)-A_Posi)+ 512)/1024)-_Vt1;        
          }else if((Title[7][0].Value==1)||((Title[TRIGG][SOURCE].Value==4)&&(TrigSourceEnable==1))){
            b=_Vt2-(Kb1[_B_Range]+B_Posi+(Kb2[_B_Range]*(V[B]-B_Posi)+ 512)/1024);
	    a=(Kb1[_B_Range]+B_Posi+(Kb2[_B_Range]*((((DataBuf[TriggerPosition-1]>>8)&0xFF)-ADCoffset)-B_Posi)+ 512)/1024)-_Vt2;        
          }else goto bypasscalculate; 
        }
        Shift=-((((b*102400)/((a+b)*TempKp1))+50)/100);		//calculate jitter from analog channels
      }
bypasscalculate:								   //digital channels		

     if ((i==155)&&(_T_base > 16)&&(_Mode!=SCAN)&&(__Get(FIFO_START)!=0)){	   //calculate jitter position if trig from dig channels
        if((Title[7][1].Value==1)||(Title[7][1].Value==3)||(Title[7][1].Value==4)||(Title[7][1].Value==5)){	//positive transitions
          if (Title[7][0].Value==2){
            for (jj=DTriggerPosition;jj<155;jj++){					
	      if (((DataBuf[jj]>>16)&1)==1){Shift=((jj-DTriggerPosition)*1024)/TempKp1;break;}       
            }		
          }else if(Title[7][0].Value==3){
            for (jj=DTriggerPosition;jj<155;jj++){
	      if (((DataBuf[jj]>>16)&2)==2){Shift=((jj-DTriggerPosition)*1024)/TempKp1;break;}
            }		
          }
        }else{										//negative transitions
          if (Title[7][0].Value==2){										
            for (jj=DTriggerPosition;jj<155;jj++){
	      if (((DataBuf[jj]>>16)&1)==0){Shift=((jj-DTriggerPosition)*1024)/TempKp1;break;}
            }		
          }else if(Title[7][0].Value==3){
            for (jj=DTriggerPosition;jj<155;jj++){
	      if (((DataBuf[jj]>>16)&2)==0){Shift=((jj-DTriggerPosition)*1024)/TempKp1;break;}
            }		
          } 
        }
      }  

      if(i >= k){                          // pointer to reach the specified window position
					   // Use V[x] loaded above rather than unsigned Ch[x], prevents clipping.
	  if (CalFlag>0) {
          Tmp=Ka2[_A_Range]*(V[A]-A_Posi)  ;        // Factor in gain correction for signal (Ka2) from signal zero point
          if (Tmp>0)Tmp+=512; else Tmp-=512;	    // Flip bias if negative, allows proper compensation of negative values
          V[A]  = Ka1[_A_Range]+A_Posi+(Tmp/1024);      
          Tmp=Kb2[_B_Range]*(V[B]-B_Posi);
          if (Tmp>0)Tmp+=512; else Tmp-=512;	   
          V[B]  = Kb1[_B_Range]+B_Posi+(Tmp/1024);     
	  }else{											    
          V[A]  = Ka1[_A_Range]+V[A];                     // gain correction switched off
          V[B]  = Kb1[_B_Range]+V[B];      
        }

        while(j > 0 ){
          if ((Options&2)&&(_T_base<17)) {   // prevent interpolate routine from interfering with non-interpolated time bases
            Send_Data( V[A_],V[B_],C_D,n++);	  // allows proper display of narrow pulses		
          }else{
            Send_Data( V[A_]+((V[A]-V[A_])*(1024 - j))/1024, // the current CH_A point interpolation
                       V[B_]+((V[B]-V[B_])*(1024 - j))/1024, // the current CH_B point interpolation
                       C_D,                                  // current point digital channel values
                       n++);
          }					
          j -= TempKp1;
          if(n >= X_SIZE){ k = 8192;  break;}    	
        }  

        j += 1024;                              
        V[A_] = V[A];  V[B_] = V[B];            

      }

    }						// end of for loop

    if ((exitflag==1)&&(JumpCnt<(393+Xtend))) exitflag=0;      //if exitflag was on at start, full frame finished allows freerun     

    if ((FrameMode!=0) && (_T_base>11) && (_Mode==SCAN)) __Set(FIFO_CLR, W_PTR);

        if(Options&4){ 	//alarms
          if(_Mode!=X_Y){

                      if(Title[V_VERNIE][V1].Value<200){	//lock out if at extreme top
			h=a_Max-A_Posi;
	                h = (((Ka2[_A_Range]*h)+512)/1024);
                        h=h+_1_posi;	 	        	
                        if((h>Title[V_VERNIE][V1].Value)&&(Title[0][0].Value!=0)&&(Title[V_VERNIE][2].Value==0)) Beeper(500);

			h=b_Max-B_Posi;
	                h = (((Kb2[_B_Range]*h)+512)/1024);
                        h=h+_2_posi;	 	        	
                        if((h>Title[V_VERNIE][V1].Value)&&(Title[1][0].Value!=0)&&(Title[V_VERNIE][2].Value==1))Beeper(500);
                      }

                      if(Title[V_VERNIE][V2].Value>0){	//lock out if at extreme bottom
			h=a_Min-A_Posi;
	                h = (((Ka2[_A_Range]*h)+512)/1024);
                        h=h+_1_posi;	 	        	
                        if((h<Title[V_VERNIE][V2].Value)&&(Title[0][0].Value!=0)&&(Title[V_VERNIE][2].Value==0))Beeper(500);

			h=b_Min-B_Posi;
	                h = (((Kb2[_B_Range]*h)+512)/1024);
                        h=h+_2_posi;	 	        	
                        if((h<Title[V_VERNIE][V2].Value)&&(Title[1][0].Value!=0)&&(Title[V_VERNIE][2].Value==1))Beeper(500);
                      }
          }

        }

  	AWavCenter=(a_Max + a_Min)/2;
        BWavCenter=(b_Max + b_Min)/2;

	if(CurDefTime>0){  		           //allow V cursors to adjust time meters trigger level
  	  h=AWavCenter;
          Tmp=h; 
          h=(((Ka2[_A_Range]*(h-A_Posi))+512)/1024)+_1_posi;
          if(Title[V_VERNIE][V1].Value<h){
            AWavCenter-=(((1024*(h-Title[V_VERNIE][V1].Value))+((512*Ka2[_A_Range])/1024))/Ka2[_A_Range]);
            if(Title[V_VERNIE][V2].Value>Title[V_VERNIE][V1].Value){
              Tmp+=(((1024*(Title[V_VERNIE][V2].Value-h))+((512*Ka2[_A_Range])/1024))/Ka2[_A_Range]);
              AWavCenter=Tmp; 
            }
          }else if(Title[V_VERNIE][V2].Value>h){
            AWavCenter+=(((1024*(Title[V_VERNIE][V2].Value-h))+((512*Ka2[_A_Range])/1024))/Ka2[_A_Range]);
          } 
  	  h=BWavCenter;
          Tmp=h; 
          h=(((Kb2[_B_Range]*(h-B_Posi))+512)/1024)+_2_posi;
          if(Title[V_VERNIE][V1].Value<h){
            BWavCenter-=(((1024*(h-Title[V_VERNIE][V1].Value))+((512*Kb2[_B_Range])/1024))/Kb2[_B_Range]);
            if(Title[V_VERNIE][V2].Value>Title[V_VERNIE][V1].Value){
              Tmp+=(((1024*(Title[V_VERNIE][V2].Value-h))+((512*Kb2[_B_Range])/1024))/Kb2[_B_Range]);
              BWavCenter=Tmp;
            }
          }else if(Title[V_VERNIE][V2].Value>h){
            BWavCenter+=(((1024*(Title[V_VERNIE][V2].Value-h))+((512*Kb2[_B_Range])/1024))/Kb2[_B_Range]);
          }
        }
        a_Mid_H = TimeCalcHyst +AWavCenter;
        a_Mid_L = a_Mid_H - (2*TimeCalcHyst);
        b_Mid_H = TimeCalcHyst +BWavCenter;
        b_Mid_L = b_Mid_H - (2*TimeCalcHyst);

        QEnd[0]=TaS;					//ending sampling positions for quantization error correction 
        QEnd[1]=TbS;

       TaS -= Ta; TbS -= Tb; TcS -= Tc; TdS -= Td;

  if((_Status == RUN)&&(Options&4)&&(ClearHoldFlag>0)){       //clear hold if in fast mode (after first loop)
    if((_T_base > 10)||((_T_base < 11)&&((JumpCnt==get_bag_max_buf())||(freerun==1)))){ //if in slow mode, wait for frame to be done     
     if(SlowModeSkip==0){    
      if(Title[TRIGG][SOURCE].Value<4){   						//if not in ab mode
        ClearMinMax(ClearHoldFlag);
        ClearHold(ClearHoldFlag);
        ClearHoldFlag=0;
      }else{										//in ab mode
         if((TrigSourceEnable==0)&&(ClearHoldFlag&1)){
           ClearMinMax(1);
           ClearHold(1);
           ClearHoldFlag--;								//removes bit 1 (tested as set)
         }
         if((TrigSourceEnable==1)&&(ClearHoldFlag&2)){
           ClearMinMax(2);
           ClearHold(2);
           ClearHoldFlag&=0xFD;								//removes bit 2
         }
      }
     }else if(SlowModeSkip>0) SlowModeSkip--;
    } 
  }    

    if(JumpCnt==get_bag_max_buf())ClearLeadingEdge=0;             //if turned on by clear screen, reset at end of frame
    if((SumResetFlag==1)&&((JumpCnt==get_bag_max_buf())||(freerun==1)||(_Status!= RUN))){   //in slow mode wait for all necessary loops 
      EnablePaS=1;									    //      	to complete frame
      ResetSum();				//sync sum resets between meter reads and completed frames 
      SumResetFlag=0;
    }
 
        if(a_Max>Ga_Max)Ga_Max=a_Max;		//update peak from all frames into Gx_Max,Min
        if(b_Max>Gb_Max)Gb_Max=b_Max;
        if(a_Min<Ga_Min)Ga_Min=a_Min;
        if(b_Min<Gb_Min)Gb_Min=b_Min;

        if((Options&4)==4){ 			//if in hold mode, transfer to GHx_xxx since Gx gets reset every read or frame interval
          if(a_Max>GHa_Max)GHa_Max=a_Max;else if(HoldResetFlag&1)GHa_Max=a_Max;		//if max is negative, establish start point
          if(b_Max>GHb_Max)GHb_Max=b_Max;else if(HoldResetFlag&2)GHb_Max=b_Max;
          if(a_Min<GHa_Min)GHa_Min=a_Min;else if(HoldResetFlag&1)GHa_Min=a_Min;		//if min is positive, start
          if(b_Min<GHb_Min)GHb_Min=b_Min;else if(HoldResetFlag&2)GHb_Min=b_Min;
          HoldResetFlag=0;
        }else{					//if not in hold mode
          GHa_Max=Ga_Max;
          GHb_Max=Gb_Max;
          GHa_Min=Ga_Min;
          GHb_Min=Gb_Min;
        }


    if((_1_source != HIDE)||(_4_source==14)||(_4_source==15)){
           UTmp= (1000000000/TaS)*TaN;			//Calculate frequency using data from last frame
           UTmp-=QError(0,QStart[0],QEnd[0],UTmp);	//Adjust for any possible quantization error compensation
	   Average(0);					//Include all previous data transfers and average
    }
    if((_2_source != HIDE)||(_4_source==12)||(_4_source==13)){          
           UTmp= (1000000000/TbS)*TbN;		
           UTmp-=QError(1,QStart[1],QEnd[1],UTmp);	
	   Average(1);
    } 	
    if(_3_source == CH_C){         
           UTmp= (1000000000/TcS)*TcN;		
	   Average(2);
    } 	
    if((_4_source == CH_D)||(_4_source == C_and_D)||(_4_source == C_or_D)){
           UTmp = (1000000000/TdS)*TdN;
	   Average(3);
    } 	

   if ((Title[TRIGG][SOURCE].Value>1)&&(Title[TRIGG][SOURCE].Value<4)){	  //if triggering from digital chs, alignment with analog chs shifts waveforms to the right,
     for(h=0; h<=(X_SIZE-5); h++){                     			  //shift back to original position    
        TrackBuff[h*4] = TrackBuff[(h+5)*4];
        TrackBuff[h*4+1] = TrackBuff[(h+5)*4+1];
        TrackBuff[h*4+2] = TrackBuff[(h+5)*4+2];
        TrackBuff[h*4+3] = TrackBuff[(h+5)*4+3];
     }
   }


if (_T_base > 16){					//jitter compensation for interpolated modes

  if(Title[TRIGG][SOURCE].Value==4){  			//if in A&B trig mode   

    if(TrigSourceEnable==0){

     if (Shift<0){
      for(h=X_SIZE;h>=-Shift; h--){                   
        TrackBuff[h*4] = TrackBuff[(h+Shift)*4];
      }
     }
     if (OldShift<0){
      for(h=X_SIZE;h>=-OldShift; h--){                   
        TrackBuff[h*4+1] = TrackBuff[(h+OldShift)*4+1];
      }
     }

    }else if(TrigSourceEnable==1){

     if (Shift<0){
      for(h=X_SIZE;h>=-Shift; h--){                   
        TrackBuff[h*4+1] = TrackBuff[(h+Shift)*4+1];
      }
     }
     if (OldShift<0){
      for(h=X_SIZE;h>=-OldShift; h--){                   
        TrackBuff[h*4] = TrackBuff[(h+OldShift)*4];
      }
     } 

    }
    
    Tmp=Shift;
    if(OldShift<Shift)Shift=OldShift;			//for blanking, use the largest neg value of the two
    OldShift=Tmp;

  }else{	//source not AltAB

    if (Shift>0){
      for(h=0; h<=(X_SIZE-Shift); h++){                   
        TrackBuff[h*4] = TrackBuff[(h+Shift)*4];
        TrackBuff[h*4+1] = TrackBuff[(h+Shift)*4+1];
        TrackBuff[h*4+2] = TrackBuff[(h+Shift)*4+2];
        TrackBuff[h*4+3] = TrackBuff[(h+Shift)*4+3];
      }
    }else if (Shift<0){
      for(h=X_SIZE;h>=-Shift; h--){                   
        TrackBuff[h*4] = TrackBuff[(h+Shift)*4];
        TrackBuff[h*4+1] = TrackBuff[(h+Shift)*4+1];
        TrackBuff[h*4+2] = TrackBuff[(h+Shift)*4+2];
        TrackBuff[h*4+3] = TrackBuff[(h+Shift)*4+3];
      }
    } 
    

  }   					//else... 

}  					//if Tbase...  


   if (_T_base < 17){			//pre signal blanking						
     if (_T_base > 10){
       discard=4;
     }else discard=1;
   }else if (Shift<0){
     discard=-Shift;
   }else discard=0;
   if (_Mode==SGL) discard=4;

							
   if (Shift>0)Rdiscard=Shift;else Rdiscard=0;		    //Shift is >0 when in highest time bases triggered from digital channel 	
   if (_T_base>16)Rdiscard+=7;else Rdiscard=0;              //and is compensated for interpolation

   for(jj=0; jj<discard; jj++){                             // Discard first pixels, fill with next valid sample
    TrackBuff[jj*4] = TrackBuff[discard*4];		    // conditional in draw.c blanks these 	
    TrackBuff[jj*4+1] = TrackBuff[discard*4+1];             // this way it doesn't misalign wave trace with trigger vernier...
   }
   if ((Title[TRIGG][SOURCE].Value<2)||(Title[TRIGG][SOURCE].Value==4))Tmp=6; else Tmp=0;      //add 6 to blank C and D if not shifting screen to the left 5 samples ie:triggering from analog channels

   for(jj=0; jj<(discard+Tmp); jj++){                       // Discard first pixels, fill with next valid sample
    TrackBuff[jj*4+2] = TrackBuff[(discard+Tmp)*4+2];          
   }

   if ((Title[TRIGG][SOURCE].Value<2)||(Title[TRIGG][SOURCE].Value==4)){				
     if ((_4_source==A_add_B)||(_4_source==A_sub_B))Tmp=0;      
   }

   for(jj=0; jj<(discard+Tmp); jj++){                           	
    TrackBuff[jj*4+3] = TrackBuff[(discard+Tmp)*4+3];
   }

   
    for(h=X_SIZE; h>(X_SIZE-Rdiscard); h--){                  //post signal blanking 
    TrackBuff[h*4] = TrackBuff[(X_SIZE-Rdiscard)*4];		     	
    TrackBuff[h*4+1] = TrackBuff[(X_SIZE-Rdiscard)*4+1];         
    TrackBuff[h*4+2] = TrackBuff[(X_SIZE-Rdiscard)*4+2];            
    TrackBuff[h*4+3] = TrackBuff[(X_SIZE-Rdiscard)*4+3];           
   }


	// FFT /////////////////////////////
	if ((ShowFFT==1) || (_4_source==SPEC_A) || (_4_source==SPEC_B)) {
	   for(i=0; i<FFTSize;i++)	fi[i] = 0;
	   fix_fft(fr, fi, FFTSize);

	   for (i=0; i < (FFTSize/2); i++)
	     {
		X= fr[i];      /* Real */			//all types are s16, except NFreq=u16
		Y= fi[i];      /* Imag */    
                fr[ i ] = Int_sqrt(X*X+Y*Y);
                if(fr[i]>198)fr[i]=198; if(fr[i]<0)fr[i]=0;      
                if((_4_source==15)||(_4_source==13)){  //wide bandwidth, sums peak with adjacent bins, so need to leave space before next peak
                  if(i>5){
                    if((fr[i-2]>=fr[i-3])&&(fr[i-2]>fr[i-1])&&(fr[i-2]>6)){	      		//detect peak (i= -2 =peak), threshold (>6)
                      fr[i-2]=Int_sqrt((fr[i-2]*fr[i-2])+(fr[i-3]*fr[i-3])+(fr[i-1]*fr[i-1]));  //sum with neighbors
                      if(fr[i-2]>198)fr[i-2]=198;			//limit to keep LCD_Buffer index within bounds (through PeakFreq)
                      fr[i-3]=2+(fr[i-2]>>3);					      		//suppress left skirt                      
                      LeftSkirt=i-3;						      		//save it's position	 
                    }
                    if((i-5)==LeftSkirt){fr[i-3]=fr[i-5];LeftSkirt=0;}  //suppress right skirt of summed peak if next bin skipped
                  }  
                  if(i>7)fr[i-6] = Exp[fr[i-6]];  //wait until bins are processed before compressing data 
                }else fr[ i ] = Exp[fr[i]];       //for normal (narrow) bandwidth

 	     }

             if((_4_source==15)||(_4_source==13))for(i=250;i<256;i++)fr[i]=Exp[fr[i]];          //compress remaining

                fr[0]=0; fr[1]=0;             

		FFTflag=1;		
		Display_Value(FRQ);	//calculate NFreq using meter routine, sets N_UNIT and ShiftDigits for proper display
		FFTflag=0;

		PeakFreq = 2;           //Called for as array index for LCD_Buffer @ -2 Less than 2 could put index out of bounds
	        imax = 0;
		for (i=2; i < FFTSize/2; i++) {
			if (PeakFreq < fr[i]) {  
				PeakFreq= fr[i] ; 
				imax = i;
			}
		}

		Int2Str(NFreqStr, NFreq,N_UNIT, 4, UNSIGN,ShiftDigits);
		Int2Str(FreqT1Str, ((NFreq / FFTBins) * _T1),N_UNIT, 4, UNSIGN,ShiftDigits);
		Int2Str(FreqDivStr, ((NFreq / FFTBins) * 30),N_UNIT, 4, UNSIGN,ShiftDigits);
	        if (imax>1)Int2Str(PeakFreqStr, ((NFreq / FFTBins) * imax),N_UNIT, 4, UNSIGN,ShiftDigits);

	}
	//////////////////////////// FFT ///
}

void Send_Data(s16 Va, s16 Vb, u8 C_D, u16 n)  // output display data
{
  s32 Tmp = 0, i;

  i = n*4;
  if(Va >= Y_BASE+Y_SIZE)  TrackBuff[i + TRACK1] = Y_BASE+Y_SIZE-1;
  else if(Va <= Y_BASE+1)  TrackBuff[i + TRACK1] = Y_BASE+1;
  else                     TrackBuff[i + TRACK1] = Va;
  if(Vb >= Y_BASE+Y_SIZE)  TrackBuff[i + TRACK2] = Y_BASE+Y_SIZE-1;
  else if(Vb <= Y_BASE+1)  TrackBuff[i + TRACK2] = Y_BASE+1;
  else                     TrackBuff[i + TRACK2] = Vb;
  
  if(C_D & 1)  TrackBuff[i + TRACK3] = c_Max;
  else         TrackBuff[i + TRACK3] = _3_posi;
  
  switch (_4_source){                       
  case A_add_B:
    Tmp = Posi_412 + Va + Vb;
    break;
  case A_sub_B:
    Tmp = Posi_41_2 + Va - Vb;	
    break;
  case C_and_D:
    //if((~C_D)& 3) Tmp = d_Max;				// was backwards... 
    //else          Tmp = _4_posi;
    if((~C_D)& 3) Tmp = _4_posi; 
    else          Tmp = d_Max;
    break;  
  case C_or_D:
    if(C_D & 3)   Tmp = d_Max; 
    else          Tmp = _4_posi;
    break;  
  case REC_1:
    Tmp = Posi_4F1 + FileBuff[n];  
    break;
  case REC_2:
    Tmp = Posi_4F2 + FileBuff[n+400];  
    break;
  case REC_3:
    Tmp = Posi_4F3 + FileBuff[n+800];  
    break;
  case REC_4:
    Tmp = Posi_4F4 +  FileBuff[n+1200];  
    break;
  case (10)...(15):		//fft functions	
    break;
  default:
    if(C_D & 2)  Tmp = d_Max;
    else         Tmp = _4_posi;
  }
  if(Tmp >= Y_BASE+Y_SIZE)  TrackBuff[i + TRACK4] = Y_BASE+Y_SIZE-1;
  else if(Tmp <= Y_BASE+1)  TrackBuff[i + TRACK4] = Y_BASE+1;
  else                      TrackBuff[i + TRACK4] = Tmp;
}
/*******************************************************************************
 Synchro: scan synchronization, waveform display by setting the mode
*******************************************************************************/
void Synchro(void)  // scan synchronization: AUTO, NORM, SGL, NONE, SCAN modes
{ 
freerun=0;
u32 waste=0;


if (BailoutFlag>0) {BailoutFlag=0; goto BailOut;} 				//if in trigger delay

if (FrameMode>0)								//single frame buffer
 {
  switch (_Mode)
  {  
     case X_Y:
     case AUTO:
        __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2)){
        if (entryflag>0){
          JumpCnt=0;
          entryflag=0;
        }
        Process();
        Wait_Cnt=shortwait[_T_base];                                              
        exitflag=1;                                                          
      }else if ((Wait_Cnt==1)&&(_T_base < 9)){
        if (_T_base <6)__Set(FIFO_CLR, W_PTR);                              
      }else if(Wait_Cnt ==0){
        if (_T_base > 8){
          freerun=1;
          entryflag=0;
        }else{
          freerun=2;						  
          entryflag=1;                                                 
        }
        if (JumpCnt > (392+Xtend))JumpCnt = 0;                     
        Process();
      }
      break;
    case NORH:
      if(Title[TRIGG][SOURCE].Value !=4) __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2)){
        Process();                                 
        Wait_Cnt=Wait[_T_base];
      }else{ 
        if(Wait_Cnt==0){
          Wait_Cnt = 1;
          //ResetSum();	                      
        }
      }
      break;
    case NORC:					
     __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2)){
          Process();
          Wait_Cnt=Wait[_T_base];
        } else if(Wait_Cnt==0)						//was ==1
         {
          ResetSum();	                      
          if (_T_base < 7) cleardatabuf( (A_Posi+ADCoffset), (B_Posi+ADCoffset),(393+Xtend),2);
          ClearTrackBuff(0);
          Wait_Cnt=Wait[_T_base];						//added
      }
      break;
   case SGL:					
      __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2))  Process();         
      break;
    case SCAN:								
      if (_T_base>11){ 
        __Set(TRIGG_MODE, UNCONDITION);            	// works best at high sweep rates
      }else{                                          // works better at low rates
       __Set(TRIGG_MODE,3);  		                  // same as track1 (0) <<3 + 3 (>vt) (do not use digital chs...) 
       __Set(V_THRESHOLD,255);                        // level then set out of range. Prevents triggering(vt>255), with fifo "splice" shifted left 150pix, gives smooth display at low speeds
      }
      if (_T_base>7){ 
        freerun=1;
        Process();                                  	
      }else{ 								//tbase<8
         freerun=2;
         if ((FlagMeter==0) && (JumpCnt> 389)) JumpCnt=0;
         if ((FlagMeter>0) && (JumpCnt> 304)) JumpCnt=0;
         Process();
      } 
   } //switch
 }
 else										//regular large buffer mode
 {
 switch (_Mode)
   { 
   case X_Y:
   case AUTO:
       __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2)){
        Process();                                 
        Wait_Cnt = Wait[_T_base];
      } else if(Wait_Cnt==0) {
          if(JumpCnt >= 4095)  JumpCnt = 0; 
          Process();   
          Wait_Cnt = Wait[_T_base];
      } break;
    case NORH:
      if(Title[TRIGG][SOURCE].Value !=4) __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2)){
        Process();                                 
        Wait_Cnt = Wait[_T_base];
      } else if(Wait_Cnt==0) {
       //ResetSum();	                      
       Wait_Cnt = Wait[_T_base];
      } break;
     case NORC:
      __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2)){
        Process();                                 
        Wait_Cnt = Wait[_T_base];
      } else if(Wait_Cnt==0) {
       ResetSum();	                      
       ClearTrackBuff(0);		      // clear screen
        Wait_Cnt = Wait[_T_base];
      } break;
   case SGL:
      __Set(TRIGG_MODE,(_Tr_source <<3)+_Tr_kind);  
      if((__Get(FIFO_START)!=0)||(_State.Value==2))  Process();         
	break;
    case SCAN:								
      if (_T_base>11){ 
        __Set(TRIGG_MODE, UNCONDITION);               	
      }else{
       __Set(TRIGG_MODE,3);  
       __Set(V_THRESHOLD,255); 
      }
      if (_T_base>7){ 
        freerun=1;
        Process();                                  
      }else{ 						 //tbase<8
         freerun=2;
         if(JumpCnt >= 4095)  JumpCnt = 0; 
         Process();
       } 
   }  										
 }   							 // end else (framemode=0)

  if (_Mode==SCAN) Wait_Cnt = 1;	     	

  if ((FrameMode > 0)&&(_Mode!=SGL)){                    // in single frame buffer mode
    if (_Mode==SCAN){
      if ((FlagMeter==0) && (JumpCnt> 389)) JumpCnt=0;
      if ((FlagMeter>0) && (JumpCnt> 304)) JumpCnt=0;
    } 
    if (JumpCnt>(392+Xtend)){
      if((__Get(FIFO_START)!=0)&&(freerun!=2)){          // resetting FIFO after start flag and frame completed provides proper triggering in single frame buffer mode
        JumpCnt=0;
        if ((Options&1)&&(_T_base < 10)&&(_Mode!=AUTO)&&(_Mode!=X_Y)&&(_Mode!=SGL)&&(_Mode!=SCAN)){	//trigger hold mode, works only with time bases where process is slower than data transfers
          CountUnread=0;
BailOut:
          if (Title[7][3].Value>0){				       //add additional adjustable delay just before resetting write pointer

            DelayLoopBreak=50;				//restrict max time in loop to 1 sec to prevent possible lockup with empty buffer
            while (CountUnread<Title[7][3].Value){

              if (__Get(FIFO_EMPTY)==0){		//if FIFO has data to read
                waste =__Read_FIFO();			//FIFO reads provide relative timing for delay
                CountUnread++;
              }else{					//while loop can lockup with empty buffer          
                if(Update>1)break;			//when initiating, drop out to reset FIFO
                if(DelayLoopBreak==0)break;      	//interrupt based breakout after 1 sec if necessary           
              }
              if ((CountUnread>0)&&((CountUnread%(TrigDelayLoop[_T_base]))==0)){    //Allow program to operate on a time base related interval while counting delay
                BailoutFlag=1;							    	
                break;
              }
            } 

          }  //if Title... (trig hold value > 0)
        }    //if options... (set trigger hold mode)

        if (BailoutFlag==0){

	    if (Title[TRIGG][SOURCE].Value==4){AlternateChannel();AltHoldoff=AltDelay[_T_base];}
            __Set(FIFO_CLR, W_PTR);			

        }

        waste++;			//compiler needs to do something with this or generates error... (used to read FIFO)
      }          //if get fifo start
    }            //if jumpcnt>
  } 		 //if framemode>0

   if(HoldOnNext==1) {
    _State.Value = HOLD;                                    	       // for single mode
    _State.Flag |= UPDAT;
    HoldOnNext=0;
    return;
   }

  if ((_Status == RUN)&&(_T_base > 10)&&(_T_base < 14)&&(_Mode != SGL)){  	    //special case for "in between" sweep rates...				
    if ((__Get(FIFO_START)!=0)&&(FrameMode >0))	{				    //prevents "double looping" in single window buffer mode
        if ((Title[TRIGG][SOURCE].Value==4)&&(_Status == RUN)) AlternateChannel();  //causing problems in 50, 100, and 200uS/div ranges
        __Set(FIFO_CLR, W_PTR);	 					  
    }          								  		
  }

  if ((_Status == RUN)&&(__Get(FIFO_FULL)!=0))  {                        // FIFO is full 

    if(_Mode != SGL) {
      if (Title[TRIGG][SOURCE].Value==4){
        AlternateChannel();
        if(_T_base < 10)AltHoldoff=AltDelay[_T_base];
      } 
      __Set(FIFO_CLR, W_PTR);                                            // FIFO write pointer reset

    }else if(_T_base < 11){						//in SGL mode, added reset after moving to slow mode
      __Set(FIFO_CLR, W_PTR);                                           //also added reset JumpCnt on hold/run reset 
    }


    if (FrameMode == 0)  Wait_Cnt = Wait[_T_base];
    JumpCnt =0;
    if(_Mode == SGL){
      HoldOnNext=1;
    }
  }

  if ((_Mode != AUTO)||(_T_base > 8)||(exitflag==0)||(entryflag==0)) Draw_Window(); //Executing time consuming screen update AFTER resetting FIFO rather than before
                                                                                    //allows enough time for START flag to register at fast time bases - fixes triggering
										    //issue of slow/random pulses not triggering

  if ((_Status == RUN)&&(__Get(FIFO_START)==0)&&(Title[TRIGG][SOURCE].Value==4)){   //alternate channels after delay if selected ch not triggered	
     if(_T_base > 9){

         cleardatabuf((A_Posi+ADCoffset),(B_Posi+ADCoffset), get_bag_max_buf(),TrigSourceEnable);
         AlternateChannel();
         __Set(FIFO_CLR, W_PTR);

     }else{
       if(AltHoldoff==0){
         AltHoldoff=AltDelay[_T_base];

         cleardatabuf((A_Posi+ADCoffset),(B_Posi+ADCoffset), get_bag_max_buf(),TrigSourceEnable);
         AlternateChannel();
         __Set(FIFO_CLR, W_PTR);

       }
       if(AltHoldoff>0)AltHoldoff--;
     }
  }

}   										               

u16 get_bag_max_buf(void) {
u16 out = 4096;

  if ((FrameMode>0)&&(_Mode!=SGL)){
    if (_Mode==SCAN){					
        if (FlagMeter==0) out =  390;
        if (FlagMeter>0) out =  305;
        if (_T_base>9) out=390+150;	    //moves trigger splice out of view to left edge of screen at fast time bases for cleaner display
    }else if (_Mode==X_Y){
        out = 393+_X_posi.Value;     
    }else{	
        out =  (393+Xtend);		    //modify buffer size if meters are on or off or XPOS shifted
    }
  }
  return out;
}	

void Beeper(u16 ms){
  if(Beep_mS<20) __Set(BEEP_VOLUME, 5*(Title[VOLUME][CLASS].Value-1)); //don't call beeper if currently beeping, seems to shut it off...
  Beep_mS = ms;
}

void AlternateChannel(void){
      	      if(TrigSourceEnable==1)TrigSourceEnable=0; else TrigSourceEnable=1;		     //cycle to next channel
	
	      if(TrigSourceEnable==0){								     //set up ch if enabled	
                if(Title[0][0].Value==0){							     //otherwise skip to next ch	
                  TrigSourceEnable=1;
                }else{
                  ReverseBitMask=0xFFFFFF00;
                  BitMask=0xFF;
                }    
              }
	      if(TrigSourceEnable==1){
                if(Title[1][0].Value==0){
                  TrigSourceEnable=0;
                }else{
                   ReverseBitMask= 0xFFFF00FF;
                  BitMask=0xFF00;	
                }    
              }

              Update_Trig(0);			
}


void cleardatabuf(u8 Afill, u8 Bfill, u16 n, u8 service)   //Clears/replaces n samples in DataBuf
{
						   
 u32 sample=0;
 u16 i;

  sample=Bfill;					 
  sample <<= 8;

  if(service==0){
    sample=Afill; 
    for (i=0; i<n; i++){
      DataBuf[i]&=0xFFFFFF00;      
      DataBuf[i]|=sample;
    }
  }

  if(service==1){
    for (i=0; i<n; i++){
      DataBuf[i]&=0xFFFF00FF;      
      DataBuf[i]|=sample;
    }
  }

  if(service==2){
    sample|=Afill;
    for (i=0; i<n; i++) DataBuf[i]=sample;      
  } 
}


void TransferFIFO(s16 i,u8 service){
  u32 swap;
	    if(service){
              DataBuf[i] &= ReverseBitMask;		//preserve previous data on alternate channel
              DataBuf[i] |= (__Read_FIFO() & BitMask);	//only load triggering channel if in alternate mode
	    } else DataBuf[i] = __Read_FIFO(); 		//normal mode

            if((TrigSourceEnable==1)||(service==0)){    //only swap bits if loading chB 
	      swap=0x300;
	      swap &= DataBuf[i];
	      if ((swap==0x100)||(swap==0x200))DataBuf[i]^=0x300; //swap 2 least significant bits of chB, fixes error in FPGA programming (V2.61)
            }
}

void SetOffset(u8 channel,u8 range, s16 Ypos) {
s32 temp;
      if(channel==0){
	temp=(1024+Ka3[range])*(Ypos-ScalingOffset);			//set Ka3 zero axis, allows scaling optimisation
        if(temp>0)temp+=512; else temp-=512;
        temp=(temp/1024)+ADCoffset+ScalingOffset;
	if (temp<0)temp=0;						//should not go anywhere close to 0 but just to make sure
        __Set(CH_A_OFFSET, temp);
      }else{
	temp=(1024+Kb3[range])*(Ypos-ScalingOffset);
        if(temp>0)temp+=512; else temp-=512;
        temp=(temp/1024)+ADCoffset+ScalingOffset;
	if (temp<0)temp=0;
        __Set(CH_B_OFFSET, temp);
      }  
}

void BatLevelCompensation(void){
u8 i;								//make sure both sets are available or don't use
 if((LoBatLevel[0]>2800)&&(LoBatLevel[0]<5200)&&(HiBatLevel[0]>2800)&&(HiBatLevel[0]<5200)){  	         		
  for(i=0;i<10;i++)Ka1[i]=InterpolateS8(0,LKa1[i],HKa1[i]);
  for(i=0;i<10;i++)Ka2[i]=InterpolateU16(0,LKa2[i],HKa2[i]);
  for(i=0;i<10;i++)Ka3[i]=InterpolateS8(0,LKa3[i],HKa3[i]);
 }
 if((LoBatLevel[1]>2800)&&(LoBatLevel[1]<5200)&&(HiBatLevel[1]>2800)&&(HiBatLevel[1]<5200)){	         		
  for(i=0;i<10;i++)Kb1[i]=InterpolateS8(1,LKb1[i],HKb1[i]);
  for(i=0;i<10;i++)Kb2[i]=InterpolateU16(1,LKb2[i],HKb2[i]);
  for(i=0;i<10;i++)Kb3[i]=InterpolateS8(1,LKb3[i],HKb3[i]);
 } 
}

s8 InterpolateS8(u8 Ch, s8 L8,s8 H8){
s32 Tmp;
  Tmp=((__Get(V_BATTERY)-LoBatLevel[Ch])*(H8-L8));
  if(Tmp>0)Tmp+=(VDiff[Ch]/2); else Tmp-=(VDiff[Ch]/2);
  return (Tmp/VDiff[Ch])+L8;
}

u16 InterpolateU16(u8 Ch, u16 L16,u16 H16){
s32 Tmp;
  Tmp=((__Get(V_BATTERY)-LoBatLevel[Ch])*(H16-L16));
  if(Tmp>0)Tmp+=(VDiff[Ch]/2); else Tmp-=(VDiff[Ch]/2);
  return (Tmp/VDiff[Ch])+L16;
}

s32 QError(u8 Ch, u16 Start, u16 End, u32 Utmp){	
s32 b,ab,tmp=0;
		//quantization error compensation; improves accuracy of frequency/period meters on analog channels

     b=(QParam(Ch,Start,1)*(QParam(Ch,End,0)+QParam(Ch,End,1)))-(QParam(Ch,End,1)*(QParam(Ch,Start,0)+QParam(Ch,Start,1)));
     ab=(QParam(Ch,Start,0)+QParam(Ch,Start,1))*(QParam(Ch,End,0)+QParam(Ch,End,1));
     tmp=Utmp/ab;
     tmp=tmp*b;
     if(Ch==0)tmp=tmp/TaS; else tmp=tmp/TbS;
     return tmp;

}

s16 QParam(u8 Ch, u16 Position,u8 service){	

          if(service){								//"b"								
            if(Ch==0){ 
              return ((DataBuf[Position]& 0xFF)-ADCoffset)-LastA_Mid;
            }else{
              return (((DataBuf[Position]>>8)&0xFF)-ADCoffset)-LastB_Mid;
            }
          }else{								//"a"
            if(Ch==0){ 
	      return FirstA_Mid-((DataBuf[Position-1]& 0xFF)-ADCoffset);        
            }else{
	      return FirstB_Mid-(((DataBuf[Position-1]>>8)&0xFF)-ADCoffset);        
            }
          }

}

void Average(u8 Ch){

              //sum averaging of all FIFO transfers; improves resolution/accuracy of all meters at faster timebases (>~5mS/div)

           if((_T_base > 10)||((JumpCnt==get_bag_max_buf())&&(_T_base>6)&&(FrameMode>0))){
             RunAvg(Ch);				//time/freq parameters
             if(Ch==0){					//voltage values
               VxAvg[0]=VRunAvg(0,a_Avg,VxAvg[0]);
               VNAvg[0]--;				//reset VN counters to do SSQ
               VxSsq[0]=VRunAvg(0,a_Ssq,VxSsq[0]);
             }else if(Ch==1){
               VxAvg[1]=VRunAvg(1,b_Avg,VxAvg[1]);
               VNAvg[1]--;
               VxSsq[1]=VRunAvg(1,b_Ssq,VxSsq[1]);
             }

             switch(Ch){				//summing for TL, TL, %duty
             case 0:
               PxS[Ch]+=PaS;             
               TxS[Ch]+=TaS;             
               TxN[Ch]+=TaN;             
               break;
             case 1:
               PxS[Ch]+=PbS;             
               TxS[Ch]+=TbS;             
               TxN[Ch]+=TbN;             
               break;
             case 2:
               PxS[Ch]+=PcS;             
               TxS[Ch]+=TcS;             
               TxN[Ch]+=TcN;             
               break;
             case 3:
               PxS[Ch]+=PdS;             
               TxS[Ch]+=TdS;             
               TxN[Ch]+=TdN;             
             }
           }else{
             Sum[Ch]=UTmp; 				//if frame rates slower than meter reads no summing possible

             switch(Ch){
             case 0:
               VxAvg[Ch]=a_Avg;
               VxSsq[Ch]=a_Ssq;
               PxS[Ch]=PaS;             
               TxS[Ch]=TaS;             
               TxN[Ch]=TaN;             
               break;
             case 1:
               VxAvg[Ch]=b_Avg;
               VxSsq[Ch]=b_Ssq;
               PxS[Ch]=PbS;             
               TxS[Ch]=TbS;             
               TxN[Ch]=TbN;             
               break;
             case 2:
               PxS[Ch]=PcS;             
               TxS[Ch]=TcS;             
               TxN[Ch]=TcN;             
               break;
             case 3:
               PxS[Ch]=PdS;             
               TxS[Ch]=TdS;             
               TxN[Ch]=TdN;             
             }
           }
}

s32 VRunAvg(u8 Ch, s32 Value, s32 Sum){ 		//running average of voltage values
       if(Value>Sum){
         Sum+=((Value-Sum)/++VNAvg[Ch]);
       }else if(Sum>Value){
         Sum-=((Sum-Value)/++VNAvg[Ch]);
       }else VNAvg[Ch]++;
       return Sum; 
}

void RunAvg(u8 Ch){
   if(UTmp>Sum[Ch]){					//moving sum average for frequency/period
     Sum[Ch]+=((UTmp-Sum[Ch])/++NAvg[Ch]);
   }else if(Sum[Ch]>UTmp){				
     Sum[Ch]-=((Sum[Ch]-UTmp)/++NAvg[Ch]);
   }else NAvg[Ch]++;         
}


void ResetSum(void){
u8 Ch;
        for(Ch=0;Ch<4;Ch++){ 
          Sum[Ch]=0;
          NAvg[Ch]=0;
        }

        if(EnablePaS){				//only reset after read, 1/second for TH,TL,%duty in large meters
          for(Ch=0;Ch<4;Ch++){ 
            PxS[Ch]=0;             
            TxS[Ch]=0;             
            TxN[Ch]=0;             
          }

        }
        VxAvg[0]=A_Posi*NSamples;
        VxAvg[1]=B_Posi*NSamples;
        for(Ch=0;Ch<2;Ch++){ 
          VxSsq[Ch]=NSamples/2;
          VNAvg[Ch]=0; 
          ClearMinMax(Ch+1);
        }

}

void ClearMinMax(u8 Channels){
  if(Channels&1){Ga_Max=(-0x7FFF);Ga_Min=0x7FFF;}       
  if(Channels&2){Gb_Max=(-0x7FFF);Gb_Min=0x7FFF;}
  if((Options&4)==0){					//if hold mode is not set, reset GHx as this is used to transfer max and min values
    ClearHold(3);
  }       
}

void ClearHold(u8 Channels){
  HoldResetFlag=Channels;
  if(Channels&1){GHa_Max=A_Posi;GHa_Min=A_Posi;}       
  if(Channels&2){GHb_Max=B_Posi;GHb_Min=B_Posi;}       
}

void ClearTrackBuff(u8 service){
u16 i;
  if(service)ClearLeadingEdge=1; else ClearLeadingEdge=0;  
  for(i=0; i<X_SIZE*4; i++)TrackBuff[i]=0;                   
}


/******************************** END OF FILE *********************************/
